#include "hc16.hpp"

//----------------------------------------------------------------------
inline ulong get_next_24bits()
{
	ulong high = ua_next_byte();
	ulong low  = ua_next_word();
	return low | (high<<16);
}

int op_get20()
{
	return 0xFFFFF & get_next_24bits();
}

//----------------------------------------------------------------------
bool decode_instr(int code,instruc_decode_t *entry)
{
	int i = 0;
	instruc_decode_t *index = InstrDecoder;
	while (index->instrnum != HC16_last)
	{
		if (index->opcode == code)
		{
			*entry = *index;
			return true;
		}
		index++;
		i++;
	}
	return false;
}

//----------------------------------------------------------------------
// analyse an instruction
int ana(void)
{
	const static int PSHM_bitfield[8] = {rD,rE,rIX,rIY,rIZ,rEK,rCCR};
	int i;

	for (i = 0; i < UA_MAXOP; i++)
		cmd.Operands[i].type = o_void;

	op_t *curop = cmd.Operands;
	int bmvalue;
	ushort code = get_byte(cmd.ea);
	code = ua_next_byte();
	instruc_decode_t idecode;
	if (!decode_instr(code,&idecode))
	{
		// doesn't match as a 1 byte opcode - try as 2 byte
		code <<= 8;
		code += ua_next_byte();
		if (!decode_instr(code,&idecode))
		{
			// unknown instruction
			msg("??? opcode\n");
			return 0;
		}
	}
	cmd.itype = idecode.instrnum;

	// byte size bit mask? this comes next - save for now
	if (idecode.bitmode == bmByte)
		bmvalue = ua_next_byte();

	// need to parse operands now
	switch(idecode.addrmode & 0xFF)
	{
		// Accumulator Offset 
	case amE_X:		// Index Register X with Accumulator E offset
		curop->type = o_reg;
		curop->reg = rE;
		curop++;
		curop->type = o_reg;
		curop->reg = rIX;
		break;
	case amE_Y:		// Index Register Y with Accumulator E offset
		curop->type = o_reg;
		curop->reg = rE;
		curop++;
		curop->type = o_reg;
		curop->reg = rIY;
		break;
	case amE_Z:		// Index Register Z with Accumulator E offset
		curop->type = o_reg;
		curop->reg = rE;
		curop++;
		curop->type = o_reg;
		curop->reg = rIZ;
		break;

		// Extended 
	case amEXT:		// Extended
		curop->type = o_near;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		curop++;
		break;
	case amEXT_EXT:		// Extended
		curop->type = o_near;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		curop++;
		curop->type = o_near;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		break;
	case amEXT20:	// 20-bit Extended
		curop->type = o_far;
		curop->addr = op_get20();
		curop->dtyp = dt_dword; // should be 20 bit
		curop++;
		break;

		// Immediate 
	case amIMM8:		// 8-bit Immediate
		// register bit mask? 
		if (idecode.bitmode == bmReg)
		{
			int regmask = ua_next_byte();
			if (idecode.instrnum == HC16_PSHM)
			{
				for (i = 0; i < 7; i++)
					if (regmask & (1 << i))
					{
						curop->type = o_reg;
						curop->reg = PSHM_bitfield[i];
						curop++;
					}
			}
			else if (idecode.instrnum == HC16_PULM)
			{
				for (i = 0; i < 7; i++)
					if (regmask & (1 << i))
					{
						curop->type = o_reg;
						curop->reg = PSHM_bitfield[6-i];
						curop++;
					}
			}
		}
		else
		{
			curop->type = o_imm;
			curop->value = ua_next_byte();
			curop->dtyp = dt_byte;
			curop++;
		}
		break;
	case amIMM16:	// 16-bit Immediate
		curop->type = o_imm;
		curop->value = ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		break;

		//Indexed 8-Bit 
	case amIND8_X:	// Index Register X with unsigned 8-bit offset
		curop->type = o_displ;
		curop->phrase = rIX;
		curop->addr = ua_next_byte();
		curop->dtyp = dt_byte;
		curop++;
		curop->type = o_reg;
		curop->reg = rIX;
		curop++;
		break;
	case amIND8_Y:	// Index Register Y with unsigned 8-bit offset
		curop->type = o_displ;
		curop->phrase = rIY;
		curop->addr = ua_next_byte();
		curop->dtyp = dt_byte;
		curop++;
		curop->type = o_reg;
		curop->reg = rIY;
		curop++;
		break;
	case amIND8_Z:	// Index Register Z with unsigned 8-bit offset
		curop->type = o_displ;
		curop->phrase = rIZ;
		curop->addr = ua_next_byte();
		curop->dtyp = dt_byte;
		curop++;
		curop->type = o_reg;
		curop->reg = rIZ;
		curop++;
		break;

		// Indexed 16-Bit
	case amIND16_X:	// Index Register X with signed 16-bit offset
		curop->type = o_displ;
		curop->phrase = rIX;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		curop->type = o_reg;
		curop->reg = rIX;
		break;
	case amIND16_Y:	// Index Register Y with signed 16-bit offset
		curop->type = o_displ;
		curop->phrase = rIY;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		curop->type = o_reg;
		curop->reg = rIY;
		curop++;
		break;
	case amIND16_Z:	// Index Register Z with signed 16-bit offset
		curop->type = o_displ;
		curop->phrase = rIZ;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		curop->type = o_reg;
		curop->reg = rIZ;
		curop++;
		break;

		// Indexed 20-Bit 
	case amIND20_X:	// Index Register X with signed 20-bit offset
		curop->type = o_displ;
		curop->phrase = rIX;
		curop->addr = op_get20();
		curop->dtyp = dt_dword;
		curop++;
		curop->type = o_reg;
		curop->reg = rIX;
		curop++;
		break;
	case amIND20_Y:	// Index Register Y with signed 20-bit offset
		curop->type = o_displ;
		curop->phrase = rIY;
		curop->addr = op_get20();
		curop->dtyp = dt_dword;
		curop++;
		curop->type = o_reg;
		curop->reg = rIY;
		curop++;
		break;
	case amIND20_Z:	// Index Register Z with signed 20-bit offset
		curop->type = o_displ;
		curop->phrase = rIZ;
		curop->addr = op_get20();
		curop->dtyp = dt_dword;
		curop++;
		curop->type = o_reg;
		curop->reg = rIZ;
		curop++;
		break;

		// Inherent 
	case amINH:		// Inherent
		break;

		// Post-modified Index 
	case amIXP:		// Signed 8-bit offset added to Index Register X after effective address is used
	case amIXP_EXT:		// Signed 8-bit offset added to Index Register X after effective address is used
		curop->type = o_phrase;
		curop->phrase = rIX;
		curop->specflag1 = op_postinc;
		curop->specflag2 = ua_next_byte();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		curop++;
		curop->type = o_near;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		curop++;
		break;
	case amEXT_IXP:		// Signed 8-bit offset added to Index Register X after effective address is used
		curop->type = o_near;
		curop->addr = ua_next_word();
		curop->dtyp = dt_word; // todo: needs to be combined with EK for real addy
		curop++;
		curop->type = o_phrase;
		curop->phrase = rIX;
		curop->specflag1 = op_postinc;
		curop->specflag2 = ua_next_byte();
		curop++;
		break;

		// Relative 
	case amREL8:		// 8-bit relative
		curop->type = o_near;
		curop->addr = cmd.ip + 6 + (char)ua_next_byte();
		curop->dtyp = dt_byte;
		curop++;
		break;
	case amREL16:		// 16-bit relative
		curop->type = o_near;
		curop->addr = cmd.ip + 6 + (short)ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		break;
	default:
		break;
	}

	// parse second operand (if needed)
	switch(idecode.addrmode >> 8)
	{
		// Relative 
	case amREL8:		// 8-bit relative
		curop->type = o_near;
		curop->addr = cmd.ip + 6 + (char)ua_next_byte();
		curop->dtyp = dt_byte;
		curop++;
		break;
	case amREL16:		// 16-bit relative  
		curop->type = o_near;
		curop->addr = cmd.ip + 6 + (short)ua_next_word();
		curop->dtyp = dt_word;
		curop++;
		break;
	default:
		break;
	} 

	// word size bit mask? this comes next
	if (idecode.bitmode == bmWord)
	{
		curop->type = o_imm;
		curop->value = ua_next_word();
		curop->dtyp = dt_word;
		curop++;
	}
	// byte size bit mask? 
	else if (idecode.bitmode == bmByte)
	{
		curop->type = o_imm;
		curop->value = bmvalue;
		curop->dtyp = dt_byte;
		curop++;
	}

	return cmd.size;
}
