#ifndef _HC16_HPP
#define _HC16_HPP

#include "windows.h"
#include "../idaidp.hpp"
#include "ins.hpp"
#include <diskio.hpp>


#define PLFM_MC68HC16     0x8044      // Motorola 68HC16
//------------------------------------------------------------------------
// customization of the 'cmd' structure:

// ash.uflag bit meanings:

#define UAS_PSAM        0x0001          // PseudoSam: use funny form of
                                        // equ for intmem
#define UAS_SECT        0x0002          // Segments are named .SECTION
#define UAS_NOSEG       0x0004          // No 'segment' directives
#define UAS_NOBIT       0x0008          // No bit.# names, use bit_#
#define UAS_SELSG       0x0010          // Segment should be selected by its name
#define UAS_EQCLN       0x0020          // ':' in EQU directives
#define UAS_AUBIT       0x0040          // Don't use BIT directives -
                                        // assembler generates bit names itself
#define UAS_CDSEG       0x0080          // Only DSEG,CSEG,XSEG
#define UAS_NODS        0x0100          // No .DS directives in Code segment
#define UAS_NOENS       0x0200          // don't specify start addr in the .end directive
#define UAS_PBIT        0x0400          // assembler knows about predefined bits
#define UAS_PBYTNODEF   0x0800          // do not define predefined byte names

enum processor_subtype_t
{
  prc_68HC16 = 0,                      // plain 68HC16
};

extern processor_subtype_t ptype;
extern char device[];
extern char deviceparams[];

extern ea_t intmem;               // address of the internal memory

ea_t map_addr(ulong off, int opnum, bool isdata);

//------------------------------------------------------------------------
// Registers
enum hc16_registers
{
	rA,
	rAM,
	rB,
	rCCR,
	rD,
	rE,
	rIR,
	rHR,
	rPC,
	rSL,
	rSP,
	rXMSK,
	rYMSK,

	// segment / index registers

	rEK,
	rIX,
	rIY,
	rIZ,
	rPK,
	rSK,
	rXK,
	rYK,
	rZK,
	rVcs,
	rVds,            // these 2 registers are required by the IDA kernel
};

// operande phrase types
enum hc16_phrasetypes
{
	op_postinc,
};

enum hc16_addrmodes
{
	// Accumulator Offset 
	amE_X,		// Index Register X with Accumulator E offset
	amE_Y,		// Index Register Y with Accumulator E offset
	amE_Z,		// Index Register Z with Accumulator E offset
	// Extended 
	amEXT,		// Extended
	amEXT_EXT,		// Extended
	amEXT20,	// 20-bit Extended
	// Immediate 
	amIMM8,		// 8-bit Immediate
	amIMM16,	// 16-bit Immediate
	//Indexed 8-Bit 
	amIND8_X,	// Index Register X with unsigned 8-bit offset
	amIND8_Y,	// Index Register Y with unsigned 8-bit offset
	amIND8_Z,	// Index Register Z with unsigned 8-bit offset
	// Indexed 16-Bit
	amIND16_X,	// Index Register X with signed 16-bit offset
	amIND16_Y,	// Index Register Y with signed 16-bit offset
	amIND16_Z,	// Index Register Z with signed 16-bit offset
	// Indexed 20-Bit 
	amIND20_X,	// Index Register X with signed 20-bit offset
	amIND20_Y,	// Index Register Y with signed 20-bit offset
	amIND20_Z,	// Index Register Z with signed 20-bit offset
	// Inherent 
	amINH,		// Inherent
	// Post-modified Index 
	amIXP,		// Signed 8-bit offset added to Index Register X after effective address is used
	amIXP_EXT,		// Signed 8-bit offset added to Index Register X after effective address is used
	amEXT_IXP,		// Signed 8-bit offset added to Index Register X after effective address is used
	// Relative 
	amREL8,		// 8-bit relative
	amREL16,	// 16-bit relative
};

enum hc16_bitmodes
{
	bmNone = 0,	// not a bit addressing mode
	bmByte,		// byte mask
	bmWord,		// word mask
	bmReg,		// register mask (for PSHM/PULM
};

//------------------------------------------------------------------------
void    header(void);
void    footer(void);
void    segstart(ea_t ea);
int     ana(void);
int     emu(void);
void    out(void);
bool    outop(op_t &op);
void    hc16_data(ea_t ea);

#endif
