#include "hc16.hpp"
	
instruc_t Instructions[] = {
{ "NUL",	0							}, // nonexistent
{ "ABA",	0							}, // Add B to A 
{ "ABX",	0							}, // Add B to IX 
{ "ABY",	0							}, // Add B to IY 
{ "ABZ",	0							}, // Add B to IZ
{ "ACE",	0							}, // Add E to AM
{ "ACED",	0							}, // Add E : D to AM 
{ "ADCA",	CF_USE1 					}, // Add with Carry to A
{ "ADCB",	CF_USE1 					}, // Add with Carry to B 
{ "ADCD",	CF_USE1 					}, // Add with Carry to D 
{ "ADCE",	CF_USE1 					}, // Add with Carry to E 
{ "ADDA",	CF_USE1 					}, // Add to A 
{ "ADDB",	CF_USE1 					}, // Add to B 
{ "ADDD",	CF_USE1 					}, // Add to D 
{ "ADDE",	CF_USE1 					}, // Add to E 
{ "ADE",	0							}, // Add D to E
{ "ADX",	0							}, // Add D to IX 
{ "ADY",	0							}, // Add D to IY 
{ "ADZ",	0							}, // Add D to IZ 
{ "AEX",	0							}, // Add E to IX 
{ "AEY",	0							}, // Add E to IY 
{ "AEZ",	0							}, // Add E to IZ 
{ "AIS",	CF_USE1 					}, // Add Immediate Data to Stack Pointer
{ "AIX",	CF_USE1 					}, // Add Immediate Value to IX
{ "AIY",	CF_USE1 					}, // Add Immediate Value to IY
{ "AIZ",	CF_USE1 					}, // Add Immediate Value to IZ
{ "ANDA",	CF_USE1 					}, // AND A 
{ "ANDB",	CF_USE1 					}, // AND B 
{ "ANDD",	CF_USE1 					}, // AND D 
{ "ANDE",	CF_USE1 					}, // AND E 
{ "ANDP",	CF_USE1 					}, // AND CCR 
{ "ASL",	CF_USE1|CF_SHFT 			}, // Arithmetic Shift Left 
{ "ASLA",	CF_SHFT 					}, // Arithmetic Shift Left A 
{ "ASLB",	CF_SHFT 					}, // Arithmetic Shift Left B 
{ "ASLD",	CF_SHFT 					}, // Arithmetic Shift Left D 
{ "ASLE",	CF_SHFT 					}, // Arithmetic Shift Left E 
{ "ASLM",	CF_SHFT 					}, // Arithmetic Shift Left
{ "ASLW",	CF_USE1|CF_SHFT 			}, // Arithmetic Shift Left Word
{ "ASR",	CF_USE1|CF_SHFT 			}, // Arithmetic Shift Right 
{ "ASRA",	CF_SHFT 					}, // Arithmetic Shift Right A
{ "ASRB",	CF_SHFT 					}, // Arithmetic Shift Right B
{ "ASRD",	CF_SHFT 					}, // Arithmetic Shift Right D
{ "ASRE",	CF_SHFT 					}, // Arithmetic Shift Right E
{ "ASRM",	CF_SHFT 					}, // Arithmetic Shift Right AM
{ "ASRW",	CF_USE1|CF_SHFT 			}, // Arithmetic Shift Right Word
{ "BCC",	CF_USE1 					}, // Branch if Carry Clear 
{ "BCLR",	CF_USE1|CF_USE2|CF_CHG1 	}, // Clear Bit(s)
{ "BCLRW",	CF_USE1|CF_USE2|CF_CHG1 	}, // Clear Bit(s) in a Word
{ "BCS",	CF_USE1 					}, // Branch if Carry Set 
{ "BEQ",	CF_USE1 					}, // Branch if Equal
{ "BGE",	CF_USE1 					}, // Branch if Greater Than or Equal to Zero
{ "BGND",	0							}, // Enter Background Debug Mode If BDM enabled
{ "BGT",	CF_USE1 					}, // Branch if Greater Than Zero
{ "BHI",	CF_USE1 					}, // Branch if Higher 
{ "BITA",	CF_USE1 					}, // Bit Test A
{ "BITB",	CF_USE1 					}, // Bit Test B 
{ "BLE",	CF_USE1 					}, // Branch if Less Than or Equal to Zero
{ "BLS",	CF_USE1 					}, // Branch if Lower or Same
{ "BLT",	CF_USE1 					}, // Branch if Less Than Zero
{ "BMI",	CF_USE1 					}, // Branch if Minus 
{ "BNE",	CF_USE1 					}, // Branch if Not Equal 
{ "BPL",	CF_USE1 					}, // Branch if Plus 
{ "BRA",	CF_USE1|CF_STOP|CF_JUMP 	}, // Branch Always 
{ "BRCLR",	CF_USE1|CF_USE2|CF_USE3 	}, // Branch if Bit(s) Clear
{ "BRN",	CF_USE1 					}, // Branch Never 
{ "BRSET",	CF_USE1|CF_USE2|CF_USE3 	}, // Branch if Bit(s) Set
{ "BSET",	CF_USE1|CF_USE2|CF_CHG1 	}, // Set Bit(s) 
{ "BSETW",	CF_USE1|CF_USE2|CF_CHG1 	}, // Set Bit(s) in Word 
{ "BSR",	CF_USE1|CF_CALL 			}, // Branch to Subroutine 
{ "BVC",	CF_USE1 					}, // Branch if Overflow Clear
{ "BVS",	CF_USE1 					}, // Branch if Overflow Set
{ "CBA",	0							}, // Compare A to B 
{ "CLR",	CF_USE1|CF_CHG1 			}, // Clear a Byte in Memory
{ "CLRA",	0							}, // Clear A 
{ "CLRB",	0							}, // Clear B 
{ "CLRD",	0							}, // Clear D 
{ "CLRE",	0							}, // Clear E
{ "CLRM",	0							}, // Clear AM
{ "CLRW",	CF_USE1|CF_CHG1 			}, // Clear a Word in Memory
{ "CMPA",	CF_USE1 					}, // Compare A to Memory 
{ "CMPB",	CF_USE1 					}, // Compare B to Memory 
{ "COM",	CF_USE1|CF_CHG1 			}, // Ones Complement
{ "COMA",	0							}, // Ones Complement A
{ "COMB",	0							}, // Ones Complement B
{ "COMD",	0							}, // Ones Complement D 
{ "COME",	0							}, // Ones Complement E
{ "COMW",	CF_USE1|CF_CHG1 			}, // Ones Complement Word
{ "CPD",	CF_USE1 					}, // Compare D to Memory 
{ "CPE",	CF_USE1 					}, // Compare E to Memory 
{ "CPS",	CF_USE1 					}, // Compare Stack Pointer to Memory
{ "CPX",	CF_USE1 					}, // Compare IX to Memory
{ "CPY",	CF_USE1 					}, // Compare IY to Memory
{ "CPZ",	CF_USE1 					}, // Compare IZ to Memory
{ "DAA",	0							}, // Decimal Adjust A 
{ "DEC",	CF_USE1|CF_CHG1 			}, // Decrement Memory 
{ "DECA",	0							}, // Decrement A 
{ "DECB",	0							}, // Decrement B 
{ "DECW",	CF_USE1|CF_CHG1 			}, // Decrement Memory Word
{ "EDIV",	0							}, // Extended Unsigned Integer Divide
{ "EDIVS",	0							}, // Extended Signed Integer Divide
{ "EMUL",	0							}, // Extended Unsigned Multiply
{ "EMULS",	0							}, // Extended Signed Multiply
{ "EORA",	CF_USE1 					}, // Exclusive OR A
{ "EORB",	CF_USE1 					}, // Exclusive OR B 
{ "EORD",	CF_USE1 					}, // Exclusive OR D 
{ "EORE",	CF_USE1 					}, // Exclusive OR E 
{ "FDIV",	0							}, // Fractional Unsigned Divide
{ "FMULS",	0							}, // Fractional Signed Multiply
{ "IDIV",	0							}, // Integer Divide 
{ "INC",	CF_USE1|CF_CHG1 			}, // Increment Memory 
{ "INCA",	0							}, // Increment A
{ "INCB",	0							}, // Increment B 
{ "INCW",	CF_USE1|CF_CHG1 			}, // Increment Memory Word
{ "JMP",	CF_USE1|CF_STOP|CF_JUMP		}, // Jump
{ "JSR",	CF_USE1|CF_CALL			 	}, // Jump to Subroutine Push (PC)
{ "LBCC",	CF_USE1 					}, // Long Branch if Carry Clear
{ "LBCS",	CF_USE1 					}, // Long Branch if Carry Set
{ "LBEQ",	CF_USE1 					}, // Long Branch if Equal to Zero
{ "LBEV",	CF_USE1 					}, // Long Branch if EV Set 
{ "LBGE",	CF_USE1 					}, // Long Branch if Greater Than or Equal to Zero
{ "LBGT",	CF_USE1 					}, // Long Branch if Greater Than Zero
{ "LBHI",	CF_USE1 					}, // Long Branch if Higher 
{ "LBLE",	CF_USE1 					}, // Long Branch if Less Than or Equal to Zero
{ "LBLS",	CF_USE1 					}, // Long Branch if Lower or Same
{ "LBLT",	CF_USE1 					}, // Long Branch if Less Than Zero
{ "LBMI",	CF_USE1 					}, // Long Branch if Minus 
{ "LBMV",	CF_USE1 					}, // Long Branch if MV Set 
{ "LBNE",	CF_USE1 					}, // Long Branch if Not Equal to Zero
{ "LBPL",	CF_USE1 					}, // Long Branch if Plus 
{ "LBRA",	CF_USE1|CF_STOP|CF_JUMP		}, // Long Branch Always 
{ "LBRN",	CF_USE1 					}, // Long Branch Never 
{ "LBSR",	CF_USE1|CF_CALL				}, // Long Branch to Subroutine
{ "LBVC",	CF_USE1 					}, // Long Branch if Overflow Clear
{ "LBVS",	CF_USE1 					}, // Long Branch if Overflow Set
{ "LDAA",	CF_USE1 					}, // Load A 
{ "LDAB",	CF_USE1 					}, // Load B 
{ "LDD",	CF_USE1 					}, // Load D 
{ "LDE",	CF_USE1 					}, // Load E 
{ "LDED",	CF_USE1 					}, // Load Concatenated
{ "LDHI",	0							}, // Initialize H and I 
{ "LDS",	CF_USE1 					}, // Load SP 
{ "LDX",	CF_USE1 					}, // Load IX 
{ "LDY",	CF_USE1 					}, // Load IY 
{ "LDZ",	CF_USE1 					}, // Load IZ 
{ "LPSTOP",	0							}, // Low Power Stop If S
{ "LSR",	CF_USE1|CF_CHG1|CF_SHFT 	}, // Logical Shift Right 
{ "LSRA",	CF_SHFT 					}, // Logical Shift Right A 
{ "LSRB",	CF_SHFT 					}, // Logical Shift Right B 
{ "LSRD",	CF_SHFT 					}, // Logical Shift Right D 
{ "LSRE",	CF_SHFT 					}, // Logical Shift Right E 
{ "LSRW",	CF_USE1|CF_CHG1|CF_SHFT 	}, // Logical Shift Right Word
{ "MAC",	CF_USE1 					}, // Multiply and Accumulate Signed 16-Bit Fractions
{ "MOVB",	CF_USE1|CF_USE2|CF_CHG2 	}, // Move Byte
{ "MOVW",	CF_USE1|CF_USE2|CF_CHG2 	}, // Move Word 
{ "MUL",	0							}, // Multiply 
{ "NEG",	CF_USE1|CF_CHG1 			}, // Negate Memory 
{ "NEGA",	0							}, // Negate A 
{ "NEGB",	0							}, // Negate B 
{ "NEGD",	0							}, // Negate D 
{ "NEGE",	0							}, // Negate E 
{ "NEGW",	CF_USE1|CF_CHG1 			}, // Negate Memory Word 
{ "NOP",	0							}, // Null Operation 
{ "ORAA",	0							}, // OR A
{ "ORAB",	0							}, // OR B 
{ "ORD",	0							}, // OR D 
{ "ORE",	0							}, // OR E 
{ "ORP",	CF_USE1 					}, // 1 OR Condition Code Register
{ "PSHA",	0							}, // Push A
{ "PSHB",	0							}, // Push B 
{ "PSHM",	0		 					}, // Push Multiple Registers
{ "PSHMAC",	0							}, // Push MAC Registers [CHANGES MANY VARS]
{ "PULA",	0							}, // Pull A 
{ "PULB",	0							}, // Pull B 
{ "PULM",	0		 					}, // Pull Multiple Registers [CHANGES MANY VARS]
{ "PULMAC",	0							}, // Pull MAC State 
{ "RMAC",	CF_USE1 					}, // Repeating Multiply and Accumulate Signed 16-Bit Fractions
{ "ROL",	CF_USE1|CF_SHFT 			}, // Rotate Left 
{ "ROLA",	CF_SHFT 					}, // Rotate Left A 
{ "ROLB",	CF_SHFT 					}, // Rotate Left B 
{ "ROLD",	CF_SHFT 					}, // Rotate Left D 
{ "ROLE",	CF_SHFT 					}, // Rotate Left E 
{ "ROLW",	CF_USE1|CF_SHFT 			}, // Rotate Left Word 
{ "ROR",	CF_USE1|CF_SHFT 			}, // Rotate Right Byte 
{ "RORA",	CF_SHFT 					}, // Rotate Right A 
{ "RORB",	CF_SHFT 					}, // Rotate Right B 
{ "RORD",	CF_SHFT 					}, // Rotate Right D
{ "RORE",	CF_SHFT 					}, // Rotate Right E 
{ "RORW",	CF_SHFT 					}, // Rotate Right Word 
{ "RTI",	CF_STOP 					}, // Return from Interrupt 
{ "RTS",	CF_STOP 					}, // Return from Subroutine
{ "SBA",	0							}, // Subtract B from A 
{ "SBCA",	CF_USE1 					}, // Subtract with Carry from A
{ "SBCB",	CF_USE1 					}, // Subtract with Carry from B
{ "SBCD",	CF_USE1 					}, // Subtract with Carry from D
{ "SBCE",	CF_USE1 					}, // Subtract with Carry from E
{ "SDE",	0							}, // Subtract D from E 
{ "STAA",	CF_CHG1 					}, // Store A 
{ "STAB",	CF_CHG1 					}, // Store B 
{ "STD",	CF_CHG1 					}, // Store D 
{ "STE",	CF_CHG1 					}, // Store E 
{ "STED",	CF_CHG1 					}, // Store Concatenated D and E
{ "STS",	CF_CHG1 					}, // Store Stack Pointer 
{ "STX",	CF_CHG1 					}, // Store IX 
{ "STY",	CF_CHG1 					}, // Store IY 
{ "STZ",	CF_CHG1 					}, // Store Z 
{ "SUBA",	CF_USE1|CF_CHG1 			}, // Subtract from A 
{ "SUBB",	CF_USE1|CF_CHG1 			}, // Subtract from B 
{ "SUBD",	CF_USE1|CF_CHG1 			}, // Subtract from D 
{ "SUBE",	CF_USE1|CF_CHG1 			}, // Subtract from E 
{ "SWI",	CF_CALL 					}, // Software Interrupt 
{ "SXT",	0							}, // Sign Extend B into A
{ "TAB",	0							}, // Transfer A to B 
{ "TAP",	0							}, // Transfer A to CCR 
{ "TBA",	0							}, // Transfer B to A
{ "TBEK",	0							}, // Transfer B to EK 
{ "TBSK",	0							}, // Transfer B to SK 
{ "TBXK",	0							}, // Transfer B to XK 
{ "TBYK",	0							}, // Transfer B to YK 
{ "TBZK",	0							}, // Transfer B to ZK 
{ "TDE",	0							}, // Transfer D to E 
{ "TDMSK",	0							}, // Transfer D to XMSK : YMSK
{ "TDP",	0							}, // Transfer D to CCR 
{ "TED",	0							}, // Transfer E to D 
{ "TEDM",	0							}, // Transfer E and D to AM[31:0] Sign Extend AM
{ "TEKB",	0							}, // Transfer EK to B 
{ "TEM",	0							}, // Transfer E to AM[31:16] Sign Extend AM Clear AM LSB
{ "TMER",	0							}, // Transfer Rounded AM to E
{ "TMET",	0							}, // Transfer Truncated AM to E
{ "TMXED",	0							}, // Transfer AM to
{ "TPA",	0							}, // Transfer CCR to A 
{ "TPD",	0							}, // Transfer CCR to D 
{ "TSKB",	0							}, // Transfer SK to B 
{ "TST",	CF_USE1 					}, // Test Byte Zero or Minus
{ "TSTA",	0							}, // Test A for Zero or Minus
{ "TSTB",	0							}, // Test B for Zero or Minus
{ "TSTD",	0							}, // Test D for Zero or Minus
{ "TSTE",	0							}, // Test E for Zero or Minus
{ "TSTW",	CF_USE1 					}, // Test for Zero or Minus Word
{ "TSX",	0							}, // Transfer SP to X 
{ "TSY",	0							}, // Transfer SP to Y 
{ "TSZ",	0							}, // Transfer SP to Z 
{ "TXKB",	0							}, // Transfer XK to B 
{ "TXS",	0							}, // Transfer X to SP 
{ "TXY",	0							}, // Transfer X to Y 
{ "TXZ",	0							}, // Transfer X to Z 
{ "TYKB",	0							}, // Transfer YK to B 
{ "TYS",	0							}, // Transfer Y to SP 
{ "TYX",	0							}, // Transfer Y to X 
{ "TYZ",	0							}, // Transfer Y to Z 
{ "TZKB",	0							}, // Transfer ZK to B 
{ "TZS",	0							}, // Transfer Z to SP 
{ "TZX",	0							}, // Transfer Z to X 
{ "TZY",	0							}, // Transfer Z to Y
{ "WAI",	0							}, // Wait for Interrupt 
{ "XGAB",	0							}, // Exchange A with B 
{ "XGDE",	0							}, // Exchange D with E 
{ "XGDX",	0							}, // Exchange D with IX 
{ "XGDY",	0							}, // Exchange D with IY 
{ "XGDZ",	0							}, // Exchange D with IZ 
{ "XGEX",	0							}, // Exchange E with IX 
{ "XGEY",	0							}, // Exchange E with IY 
{ "XGEZ",	0							}, // Exchange E with IZ 
};

instruc_decode_t InstrDecoder[] = {
	{HC16_ABA, 0x370B, amINH, bmNone}, // Add B to A 
	{HC16_ABX, 0x374F, amINH, bmNone}, // Add B to IX 
	{HC16_ABY, 0x375F, amINH, bmNone}, // Add B to IY 
	{HC16_ABZ, 0x376F, amINH, bmNone}, // Add B to IZ
	{HC16_ACE, 0x3722, amINH, bmNone}, // Add E to AM
	{HC16_ACED, 0x3723, amINH, bmNone}, // Add E : D to AM 
	{HC16_ADCA, 0x1743, amIND16_X, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x1753, amIND16_Y, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x1763, amIND16_Z, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x1773, amEXT, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x2743, amE_X, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x2753, amE_Y, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x2763, amE_Z, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x43, amIND8_X, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x53, amIND8_Y, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x63, amIND8_Z, bmNone}, // Add with Carry to A
	{HC16_ADCA, 0x73, amIMM8, bmNone}, // Add with Carry to A
	{HC16_ADCB, 0x17C3, amIND16_X, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x17D3, amIND16_Y, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x17E3, amIND16_Z, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x17F3, amEXT, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x27C3, amE_X, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x27D3, amE_Y, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0x27E3, amE_Z, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0xC3, amIND8_X, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0xD3, amIND8_Y, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0xE3, amIND8_Z, bmNone}, // Add with Carry to B 
	{HC16_ADCB, 0xF3, amIMM8, bmNone}, // Add with Carry to B 
	{HC16_ADCD, 0x2783, amE_X, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x2793, amE_Y, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x27A3, amE_Z, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x37B3, amIMM16, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x37C3, amIND16_X, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x37D3, amIND16_Y, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x37E3, amIND16_Z, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x37F3, amEXT, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x83, amIND8_X, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0x93, amIND8_Y, bmNone}, // Add with Carry to D 
	{HC16_ADCD, 0xA3, amIND8_Z, bmNone}, // Add with Carry to D 
	{HC16_ADCE, 0x3733, amIMM16, bmNone}, // Add with Carry to E 
	{HC16_ADCE, 0x3743, amIND16_X, bmNone}, // Add with Carry to E 
	{HC16_ADCE, 0x3753, amIND16_Y, bmNone}, // Add with Carry to E 
	{HC16_ADCE, 0x3763, amIND16_Z, bmNone}, // Add with Carry to E 
	{HC16_ADCE, 0x3773, amEXT, bmNone}, // Add with Carry to E 
	{HC16_ADDA, 0x1741, amIND16_X, bmNone}, // Add to A 
	{HC16_ADDA, 0x1751, amIND16_Y, bmNone}, // Add to A 
	{HC16_ADDA, 0x1761, amIND16_Z, bmNone}, // Add to A 
	{HC16_ADDA, 0x1771, amEXT, bmNone}, // Add to A 
	{HC16_ADDA, 0x2741, amE_X, bmNone}, // Add to A 
	{HC16_ADDA, 0x2751, amE_Y, bmNone}, // Add to A 
	{HC16_ADDA, 0x2761, amE_Z, bmNone}, // Add to A 
	{HC16_ADDA, 0x41, amIND8_X, bmNone}, // Add to A 
	{HC16_ADDA, 0x51, amIND8_Y, bmNone}, // Add to A 
	{HC16_ADDA, 0x61, amIND8_Z, bmNone}, // Add to A 
	{HC16_ADDA, 0x71, amIMM8, bmNone}, // Add to A 
	{HC16_ADDB, 0x17C1, amIND16_X, bmNone}, // Add to B 
	{HC16_ADDB, 0x17D1, amIND16_Y, bmNone}, // Add to B 
	{HC16_ADDB, 0x17E1, amIND16_Z, bmNone}, // Add to B 
	{HC16_ADDB, 0x17F1, amEXT, bmNone}, // Add to B 
	{HC16_ADDB, 0x27C1, amE_X, bmNone}, // Add to B 
	{HC16_ADDB, 0x27D1, amE_Y, bmNone}, // Add to B 
	{HC16_ADDB, 0x27E1, amE_Z, bmNone}, // Add to B 
	{HC16_ADDB, 0xC1, amIND8_X, bmNone}, // Add to B 
	{HC16_ADDB, 0xD1, amIND8_Y, bmNone}, // Add to B 
	{HC16_ADDB, 0xE1, amIND8_Z, bmNone}, // Add to B 
	{HC16_ADDB, 0xF1, amIMM8, bmNone}, // Add to B 
	{HC16_ADDD, 0x2781, amE_X, bmNone}, // Add to D 
	{HC16_ADDD, 0x2791, amE_Y, bmNone}, // Add to D 
	{HC16_ADDD, 0x27A1, amE_Z, bmNone}, // Add to D 
	{HC16_ADDD, 0x37B1, amIMM16, bmNone}, // Add to D 
	{HC16_ADDD, 0x37C1, amIND16_X, bmNone}, // Add to D 
	{HC16_ADDD, 0x37D1, amIND16_Y, bmNone}, // Add to D 
	{HC16_ADDD, 0x37E1, amIND16_Z, bmNone}, // Add to D 
	{HC16_ADDD, 0x37F1, amEXT, bmNone}, // Add to D 
	{HC16_ADDD, 0x81, amIND8_X, bmNone}, // Add to D 
	{HC16_ADDD, 0x91, amIND8_Y, bmNone}, // Add to D 
	{HC16_ADDD, 0xA1, amIND8_Z, bmNone}, // Add to D 
	{HC16_ADDD, 0xFC, amIMM8, bmNone}, // Add to D 
	{HC16_ADDE, 0x3731, amIMM16, bmNone}, // Add to E 
	{HC16_ADDE, 0x3741, amIND16_X, bmNone}, // Add to E 
	{HC16_ADDE, 0x3751, amIND16_Y, bmNone}, // Add to E 
	{HC16_ADDE, 0x3761, amIND16_Z, bmNone}, // Add to E 
	{HC16_ADDE, 0x3771, amEXT, bmNone}, // Add to E 
	{HC16_ADDE, 0x7C, amIMM8, bmNone}, // Add to E 
	{HC16_ADE, 0x2778, amINH, bmNone}, // Add D to E
	{HC16_ADX, 0x37CD, amINH, bmNone}, // Add D to IX 
	{HC16_ADY, 0x37DD, amINH, bmNone}, // Add D to IY 
	{HC16_ADZ, 0x37ED, amINH, bmNone}, // Add D to IZ 
	{HC16_AEX, 0x374D, amINH, bmNone}, // Add E to IX 
	{HC16_AEY, 0x375D, amINH, bmNone}, // Add E to IY 
	{HC16_AEZ, 0x376D, amINH, bmNone}, // Add E to IZ 
	{HC16_AIS, 0x373F, amIMM16, bmNone}, // Add Immediate Data to Stack Pointer
	{HC16_AIS, 0x3F, amIMM8, bmNone}, // Add Immediate Data to Stack Pointer
	{HC16_AIX, 0x373C, amIMM16, bmNone}, // Add Immediate Value to IX
	{HC16_AIX, 0x3C, amIMM8, bmNone}, // Add Immediate Value to IX
	{HC16_AIY, 0x373D, amIMM16, bmNone}, // Add Immediate Value to IY
	{HC16_AIY, 0x3D, amIMM8, bmNone}, // Add Immediate Value to IY
	{HC16_AIZ, 0x373E, amIMM16, bmNone}, // Add Immediate Value to IZ
	{HC16_AIZ, 0x3E, amIMM8, bmNone}, // Add Immediate Value to IZ
	{HC16_ANDA, 0x1746, amIND16_X, bmNone}, // AND A 
	{HC16_ANDA, 0x1756, amIND16_Y, bmNone}, // AND A 
	{HC16_ANDA, 0x1766, amIND16_Z, bmNone}, // AND A 
	{HC16_ANDA, 0x1776, amEXT, bmNone}, // AND A 
	{HC16_ANDA, 0x2746, amE_X, bmNone}, // AND A 
	{HC16_ANDA, 0x2756, amE_Y, bmNone}, // AND A 
	{HC16_ANDA, 0x2766, amE_Z, bmNone}, // AND A 
	{HC16_ANDA, 0x46, amIND8_X, bmNone}, // AND A 
	{HC16_ANDA, 0x56, amIND8_Y, bmNone}, // AND A 
	{HC16_ANDA, 0x66, amIND8_Z, bmNone}, // AND A 
	{HC16_ANDA, 0x76, amIMM8, bmNone}, // AND A 
	{HC16_ANDB, 0x17C6, amIND16_X, bmNone}, // AND B 
	{HC16_ANDB, 0x17D6, amIND16_Y, bmNone}, // AND B 
	{HC16_ANDB, 0x17E6, amIND16_Z, bmNone}, // AND B 
	{HC16_ANDB, 0x17F6, amEXT, bmNone}, // AND B 
	{HC16_ANDB, 0x27C6, amE_X, bmNone}, // AND B 
	{HC16_ANDB, 0x27D6, amE_Y, bmNone}, // AND B 
	{HC16_ANDB, 0x27E6, amE_Z, bmNone}, // AND B 
	{HC16_ANDB, 0xC6, amIND8_X, bmNone}, // AND B 
	{HC16_ANDB, 0xD6, amIND8_Y, bmNone}, // AND B 
	{HC16_ANDB, 0xE6, amIND8_Z, bmNone}, // AND B 
	{HC16_ANDB, 0xF6, amIMM8, bmNone}, // AND B 
	{HC16_ANDD, 0x2786, amE_X, bmNone}, // AND D 
	{HC16_ANDD, 0x2796, amE_Y, bmNone}, // AND D 
	{HC16_ANDD, 0x27A6, amE_Z, bmNone}, // AND D 
	{HC16_ANDD, 0x37B6, amIMM16, bmNone}, // AND D 
	{HC16_ANDD, 0x37C6, amIND16_X, bmNone}, // AND D 
	{HC16_ANDD, 0x37D6, amIND16_Y, bmNone}, // AND D 
	{HC16_ANDD, 0x37E6, amIND16_Z, bmNone}, // AND D 
	{HC16_ANDD, 0x37F6, amEXT, bmNone}, // AND D 
	{HC16_ANDD, 0x86, amIND8_X, bmNone}, // AND D 
	{HC16_ANDD, 0x96, amIND8_Y, bmNone}, // AND D 
	{HC16_ANDD, 0xA6, amIND8_Z, bmNone}, // AND D 
	{HC16_ANDE, 0x3736, amIMM16, bmNone}, // AND E 
	{HC16_ANDE, 0x3746, amIND16_X, bmNone}, // AND E 
	{HC16_ANDE, 0x3756, amIND16_Y, bmNone}, // AND E 
	{HC16_ANDE, 0x3766, amIND16_Z, bmNone}, // AND E 
	{HC16_ANDE, 0x3776, amEXT, bmNone}, // AND E 
	{HC16_ANDP, 0x373A, amIMM16, bmNone}, 
	{HC16_ASL, 0x14, amIND8_Y, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x1704, amIND16_X, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x1714, amIND16_Y, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x1724, amIND16_Z, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x1734, amEXT, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x24, amIND8_Z, bmNone}, // Arithmetic Shift Left 
	{HC16_ASL, 0x4, amIND8_X, bmNone}, // Arithmetic Shift Left 
	{HC16_ASLA, 0x3704, amINH, bmNone}, // Arithmetic Shift Left A 
	{HC16_ASLB, 0x3714, amINH, bmNone}, // Arithmetic Shift Left B 
	{HC16_ASLD, 0x27F4, amINH, bmNone}, // Arithmetic Shift Left D 
	{HC16_ASLE, 0x2774, amINH, bmNone}, // Arithmetic Shift Left E 
	{HC16_ASLM, 0x27B6, amINH, bmNone}, // Arithmetic Shift Left
	{HC16_ASLW, 0x2704, amIND16_X, bmNone}, // Arithmetic Shift Left Word
	{HC16_ASLW, 0x2714, amIND16_Y, bmNone}, // Arithmetic Shift Left Word
	{HC16_ASLW, 0x2724, amIND16_Z, bmNone}, // Arithmetic Shift Left Word
	{HC16_ASLW, 0x2734, amEXT, bmNone}, // Arithmetic Shift Left Word
	{HC16_ASR, 0x0D, amIND8_X, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x170D, amIND16_X, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x171D, amIND16_Y, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x172D, amIND16_Z, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x173D, amEXT, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x1D, amIND8_Y, bmNone}, // Arithmetic Shift Right 
	{HC16_ASR, 0x2D, amIND8_Z, bmNone}, // Arithmetic Shift Right 
	{HC16_ASRA, 0x370D, amINH, bmNone}, // Arithmetic Shift Right A
	{HC16_ASRB, 0x371D, amINH, bmNone}, // Arithmetic Shift Right B
	{HC16_ASRD, 0x27FD, amINH, bmNone}, // Arithmetic Shift Right D
	{HC16_ASRE, 0x277D, amINH, bmNone}, // Arithmetic Shift Right E
	{HC16_ASRM, 0x27BA, amINH, bmNone}, // Arithmetic Shift Right AM
	{HC16_ASRW, 0x270D, amIND16_X, bmNone}, // Arithmetic Shift Right Word
	{HC16_ASRW, 0x271D, amIND16_Y, bmNone}, // Arithmetic Shift Right Word
	{HC16_ASRW, 0x272D, amIND16_Z, bmNone}, // Arithmetic Shift Right Word
	{HC16_ASRW, 0x273D, amEXT, bmNone}, // Arithmetic Shift Right Word
	{HC16_BCC, 0xB4, amREL8, bmNone}, // Branch if Carry Clear 
	{HC16_BCLR, 0x1708, amIND8_X, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x1718, amIND8_Y, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x1728, amIND8_Z, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x18, amIND16_Y, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x28, amIND16_Z, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x38, amEXT, bmByte}, // Clear Bit(s)
	{HC16_BCLR, 0x8, amIND16_X, bmByte}, // Clear Bit(s)
	{HC16_BCLRW, 0x2708, amIND16_X, bmWord}, // Clear Bit(s) in a Word
	{HC16_BCLRW, 0x2718, amIND16_Y, bmWord}, // Clear Bit(s) in a Word
	{HC16_BCLRW, 0x2728, amIND16_Z, bmWord}, // Clear Bit(s) in a Word
	{HC16_BCLRW, 0x2738, amEXT, bmWord}, // Clear Bit(s) in a Word
	{HC16_BCS, 0xB5, amREL8, bmNone}, // Branch if Carry Set 
	{HC16_BEQ, 0xB7, amREL8, bmNone}, // Branch if Equal
	{HC16_BGE, 0xBC, amREL8, bmNone}, // Branch if Greater Than or Equal to Zero
	{HC16_BGND, 0x37A6, amINH, bmNone}, // Enter Background Debug Mode If BDM enabled
	{HC16_BGT, 0xBE, amREL8, bmNone}, // Branch if Greater Than Zero
	{HC16_BHI, 0xB2, amREL8, bmNone}, // Branch if Higher 
	{HC16_BITA, 0x1749, amIND16_X, bmNone}, // Bit Test A
	{HC16_BITA, 0x1759, amIND16_Y, bmNone}, // Bit Test A
	{HC16_BITA, 0x1769, amIND16_Z, bmNone}, // Bit Test A
	{HC16_BITA, 0x1779, amEXT, bmNone}, // Bit Test A
	{HC16_BITA, 0x2749, amE_X, bmNone}, // Bit Test A
	{HC16_BITA, 0x2759, amE_Y, bmNone}, // Bit Test A
	{HC16_BITA, 0x2769, amE_Z, bmNone}, // Bit Test A
	{HC16_BITA, 0x49, amIND8_X, bmNone}, // Bit Test A
	{HC16_BITA, 0x59, amIND8_Y, bmNone}, // Bit Test A
	{HC16_BITA, 0x69, amIND8_Z, bmNone}, // Bit Test A
	{HC16_BITA, 0x79, amIMM8, bmNone}, // Bit Test A
	{HC16_BITB, 0x17C9, amIND16_X, bmNone}, // Bit Test B 
	{HC16_BITB, 0x17D9, amIND16_Y, bmNone}, // Bit Test B 
	{HC16_BITB, 0x17E9, amIND16_Z, bmNone}, // Bit Test B 
	{HC16_BITB, 0x17F9, amEXT, bmNone}, // Bit Test B 
	{HC16_BITB, 0x27C9, amE_X, bmNone}, // Bit Test B 
	{HC16_BITB, 0x27D9, amE_Y, bmNone}, // Bit Test B 
	{HC16_BITB, 0x27E9, amE_Z, bmNone}, // Bit Test B 
	{HC16_BITB, 0xC9, amIND8_X, bmNone}, // Bit Test B 
	{HC16_BITB, 0xD9, amIND8_Y, bmNone}, // Bit Test B 
	{HC16_BITB, 0xE9, amIND8_Z, bmNone}, // Bit Test B 
	{HC16_BITB, 0xF9, amIMM8, bmNone}, // Bit Test B 
	{HC16_BLE, 0xBF, amREL8, bmNone}, // Branch if Less Than or Equal to Zero
	{HC16_BLS, 0xB3, amREL8, bmNone}, // Branch if Lower or Same
	{HC16_BLT, 0xBD, amREL8, bmNone}, // Branch if Less Than Zero
	{HC16_BMI, 0xBB, amREL8, bmNone}, // Branch if Minus 
	{HC16_BNE, 0xB6, amREL8, bmNone}, // Branch if Not Equal 
	{HC16_BPL, 0xBA, amREL8, bmNone}, // Branch if Plus 
	{HC16_BRA, 0xB0, amREL8, bmNone}, // Branch Always 
	{HC16_BRCLR, 0x0A, amIND16_X+(amREL16<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0x1A, amIND16_Y+(amREL16<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0x2A, amIND16_Z+(amREL16<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0x3A, amEXT+(amREL16<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0xCB, amIND8_X+(amREL8<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0xDB, amIND8_Y+(amREL8<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRCLR, 0xEB, amIND8_Z+(amREL8<<8), bmByte}, // Branch if Bit(s) Clear
	{HC16_BRN, 0xB1, amREL8, bmNone}, // Branch Never 
	{HC16_BRSET, 0x0B, amIND16_X+(amREL16<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0x1B, amIND16_Y+(amREL16<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0x2B, amIND16_Z+(amREL16<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0x3B, amEXT+(amREL16<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0x8B, amIND8_X+(amREL8<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0x9B, amIND8_Y+(amREL8<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BRSET, 0xAB, amIND8_Z+(amREL8<<8), bmByte}, // Branch if Bit(s) Set
	{HC16_BSET, 0x1709, amIND8_X, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x1719, amIND8_Y, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x1729, amIND8_Z, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x19, amIND16_Y, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x29, amIND16_Z, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x39, amEXT, bmByte}, // Set Bit(s) 
	{HC16_BSET, 0x9, amIND16_X, bmByte}, // Set Bit(s) 
	{HC16_BSETW, 0x2709, amIND16_X, bmWord}, // Set Bit(s) in Word 
	{HC16_BSETW, 0x2719, amIND16_Y, bmWord}, // Set Bit(s) in Word 
	{HC16_BSETW, 0x2729, amIND16_Z, bmWord}, // Set Bit(s) in Word 
	{HC16_BSETW, 0x2739, amEXT, bmWord}, // Set Bit(s) in Word 
	{HC16_BSR, 0x36, amREL8, bmNone}, // Branch to Subroutine 
	{HC16_BVC, 0xB8, amREL8, bmNone}, // Branch if Overflow Clear
	{HC16_BVS, 0xB9, amREL8, bmNone}, // Branch if Overflow Set
	{HC16_CBA, 0x371B, amINH, bmNone}, // Compare A to B 
	{HC16_CLR, 0x15, amIND8_Y, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x1705, amIND16_X, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x1715, amIND16_Y, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x1725, amIND16_Z, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x1735, amEXT, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x25, amIND8_Z, bmNone}, // Clear a Byte in Memory
	{HC16_CLR, 0x5, amIND8_X, bmNone}, // Clear a Byte in Memory
	{HC16_CLRA, 0x3705, amINH, bmNone}, // Clear A 
	{HC16_CLRB, 0x3715, amINH, bmNone}, // Clear B 
	{HC16_CLRD, 0x27F5, amINH, bmNone}, // Clear D 
	{HC16_CLRE, 0x2775, amINH, bmNone}, // Clear E
	{HC16_CLRM, 0x27B7, amINH, bmNone}, // Clear AM
	{HC16_CLRW, 0x2705, amIND16_X, bmNone}, // Clear a Word in Memory
	{HC16_CLRW, 0x2715, amIND16_Y, bmNone}, // Clear a Word in Memory
	{HC16_CLRW, 0x2725, amIND16_Z, bmNone}, // Clear a Word in Memory
	{HC16_CLRW, 0x2735, amEXT, bmNone}, // Clear a Word in Memory
	{HC16_CMPA, 0x1748, amIND16_X, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x1758, amIND16_Y, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x1768, amIND16_Z, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x1778, amEXT, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x2748, amE_X, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x2758, amE_Y, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x2768, amE_Z, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x48, amIND8_X, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x58, amIND8_Y, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x68, amIND8_Z, bmNone}, // Compare A to Memory 
	{HC16_CMPA, 0x78, amIMM8, bmNone}, // Compare A to Memory 
	{HC16_CMPB, 0x17C8, amIND16_X, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x17D8, amIND16_Y, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x17E8, amIND16_Z, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x17F8, amEXT, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x27C8, amE_X, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x27D8, amE_Y, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0x27E8, amE_Z, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0xC8, amIND8_X, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0xD8, amIND8_Y, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0xE8, amIND8_Z, bmNone}, // Compare B to Memory 
	{HC16_CMPB, 0xF8, amIMM8, bmNone}, // Compare B to Memory 
	{HC16_COM, 0x0, amIND8_X, bmNone}, // Ones Complement
	{HC16_COM, 0x10, amIND8_Y, bmNone}, // Ones Complement
	{HC16_COM, 0x1700, amIND16_X, bmNone}, // Ones Complement
	{HC16_COM, 0x1710, amIND16_Y, bmNone}, // Ones Complement
	{HC16_COM, 0x1720, amIND16_Z, bmNone}, // Ones Complement
	{HC16_COM, 0x1730, amEXT, bmNone}, // Ones Complement
	{HC16_COM, 0x20, amIND8_Z, bmNone}, // Ones Complement
	{HC16_COMA, 0x3700, amINH, bmNone}, // Ones Complement A
	{HC16_COMB, 0x3710, amINH, bmNone}, // Ones Complement B
	{HC16_COMD, 0x27F0, amINH, bmNone}, // Ones Complement D 
	{HC16_COME, 0x2770, amINH, bmNone}, // Ones Complement E
	{HC16_COMW, 0x2700, amIND16_X, bmNone}, // Ones Complement Word
	{HC16_COMW, 0x2710, amIND16_Y, bmNone}, // Ones Complement Word
	{HC16_COMW, 0x2720, amIND16_Z, bmNone}, // Ones Complement Word
	{HC16_COMW, 0x2730, amEXT, bmNone}, // Ones Complement Word
	{HC16_CPD, 0x2788, amE_X, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x2798, amE_Y, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x27A8, amE_Z, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x37B8, amIMM16, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x37C8, amIND16_X, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x37D8, amIND16_Y, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x37E8, amIND16_Z, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x37F8, amEXT, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x88, amIND8_X, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0x98, amIND8_Y, bmNone}, // Compare D to Memory 
	{HC16_CPD, 0xA8, amIND8_Z, bmNone}, // Compare D to Memory 
	{HC16_CPE, 0x3738, amIMM16, bmNone}, // Compare E to Memory 
	{HC16_CPE, 0x3748, amIND16_X, bmNone}, // Compare E to Memory 
	{HC16_CPE, 0x3758, amIND16_Y, bmNone}, // Compare E to Memory 
	{HC16_CPE, 0x3768, amIND16_Z, bmNone}, // Compare E to Memory 
	{HC16_CPE, 0x3778, amEXT, bmNone}, // Compare E to Memory 
	{HC16_CPS, 0x174F, amIND16_X, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x175F, amIND16_Y, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x176F, amIND16_Z, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x177F, amEXT, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x377F, amIMM16, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x4F, amIND8_X, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x5F, amIND8_Y, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPS, 0x6F, amIND8_Z, bmNone}, // Compare Stack Pointer to Memory
	{HC16_CPX, 0x174C, amIND16_X, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x175C, amIND16_Y, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x176C, amIND16_Z, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x177C, amEXT, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x377C, amIMM16, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x4C, amIND8_X, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x5C, amIND8_Y, bmNone}, // Compare IX to Memory
	{HC16_CPX, 0x6C, amIND8_Z, bmNone}, // Compare IX to Memory
	{HC16_CPY, 0x174D, amIND16_X, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x175D, amIND16_Y, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x176D, amIND16_Z, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x177D, amEXT, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x377D, amIMM16, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x4D, amIND8_X, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x5D, amIND8_Y, bmNone}, // Compare IY to Memory
	{HC16_CPY, 0x6D, amIND8_Z, bmNone}, // Compare IY to Memory
	{HC16_CPZ, 0x174E, amIND16_X, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x175E, amIND16_Y, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x176E, amIND16_Z, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x177E, amEXT, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x377E, amIMM16, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x4E, amIND8_X, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x5E, amIND8_Y, bmNone}, // Compare IZ to Memory
	{HC16_CPZ, 0x6E, amIND8_Z, bmNone}, // Compare IZ to Memory
	{HC16_DAA, 0x3721, amINH, bmNone}, // Decimal Adjust A 
	{HC16_DEC, 0x1, amIND8_X, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x11, amIND8_Y, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x1701, amIND16_X, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x1711, amIND16_Y, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x1721, amIND16_Z, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x1731, amEXT, bmNone}, // Decrement Memory 
	{HC16_DEC, 0x21, amIND8_Z, bmNone}, // Decrement Memory 
	{HC16_DECA, 0x3701, amINH, bmNone}, // Decrement A 
	{HC16_DECB, 0x3711, amINH, bmNone}, // Decrement B 
	{HC16_DECW, 0x2701, amIND16_X, bmNone}, // Decrement Memory Word
	{HC16_DECW, 0x2711, amIND16_Y, bmNone}, // Decrement Memory Word
	{HC16_DECW, 0x2721, amIND16_Z, bmNone}, // Decrement Memory Word
	{HC16_DECW, 0x2731, amEXT, bmNone}, // Decrement Memory Word
	{HC16_EDIV, 0x3728, amINH, bmNone}, // Extended Unsigned Integer Divide
	{HC16_EDIVS, 0x3729, amINH, bmNone}, // Extended Signed Integer Divide
	{HC16_EMUL, 0x3725, amINH, bmNone}, // Extended Unsigned Multiply
	{HC16_EMULS, 0x3726, amINH, bmNone}, // Extended Signed Multiply
	{HC16_EORA, 0x1744, amIND16_X, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x1754, amIND16_Y, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x1764, amIND16_Z, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x1774, amEXT, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x2744, amE_X, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x2754, amE_Y, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x2764, amE_Z, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x44, amIND8_X, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x54, amIND8_Y, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x64, amIND8_Z, bmNone}, // Exclusive OR A
	{HC16_EORA, 0x74, amIMM8, bmNone}, // Exclusive OR A
	{HC16_EORB, 0x17C4, amIND16_X, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x17D4, amIND16_Y, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x17E4, amIND16_Z, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x17F4, amEXT, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x27C4, amE_X, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x27D4, amE_Y, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0x27E4, amE_Z, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0xC4, amIND8_X, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0xD4, amIND8_Y, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0xE4, amIND8_Z, bmNone}, // Exclusive OR B 
	{HC16_EORB, 0xF4, amIMM8, bmNone}, // Exclusive OR B 
	{HC16_EORD, 0x2784, amE_X, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x2794, amE_Y, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x27A4, amE_Z, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x37B4, amIMM16, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x37C4, amIND16_X, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x37D4, amIND16_Y, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x37E4, amIND16_Z, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x37F4, amEXT, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x84, amIND8_X, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0x94, amIND8_Y, bmNone}, // Exclusive OR D 
	{HC16_EORD, 0xA4, amIND8_Z, bmNone}, // Exclusive OR D 
	{HC16_EORE, 0x3734, amIMM16, bmNone}, // Exclusive OR E 
	{HC16_EORE, 0x3744, amIND16_X, bmNone}, // Exclusive OR E 
	{HC16_EORE, 0x3754, amIND16_Y, bmNone}, // Exclusive OR E 
	{HC16_EORE, 0x3764, amIND16_Z, bmNone}, // Exclusive OR E 
	{HC16_EORE, 0x3774, amEXT, bmNone}, // Exclusive OR E 
	{HC16_FDIV, 0x372B, amINH, bmNone}, // Fractional Unsigned Divide
	{HC16_FMULS, 0x3727, amINH, bmNone}, // Fractional Signed Multiply
	{HC16_IDIV, 0x372A, amINH, bmNone}, // Integer Divide 
	{HC16_INC, 0x13, amIND8_Y, bmNone}, // Increment Memory 
	{HC16_INC, 0x1703, amIND16_X, bmNone}, // Increment Memory 
	{HC16_INC, 0x1713, amIND16_Y, bmNone}, // Increment Memory 
	{HC16_INC, 0x1723, amIND16_Z, bmNone}, // Increment Memory 
	{HC16_INC, 0x1733, amEXT, bmNone}, // Increment Memory 
	{HC16_INC, 0x23, amIND8_Z, bmNone}, // Increment Memory 
	{HC16_INC, 0x3, amIND8_X, bmNone}, // Increment Memory 
	{HC16_INCA, 0x3703, amINH, bmNone}, // Increment A
	{HC16_INCB, 0x3713, amINH, bmNone}, // Increment B 
	{HC16_INCW, 0x2703, amIND16_X, bmNone}, // Increment Memory Word
	{HC16_INCW, 0x2713, amIND16_Y, bmNone}, // Increment Memory Word
	{HC16_INCW, 0x2723, amIND16_Z, bmNone}, // Increment Memory Word
	{HC16_INCW, 0x2733, amEXT, bmNone}, // Increment Memory Word
	{HC16_JMP, 0x4B, amIND20_X, bmNone}, // Jump
	{HC16_JMP, 0x5B, amIND20_Y, bmNone}, // Jump
	{HC16_JMP, 0x6B, amIND20_Z, bmNone}, // Jump
	{HC16_JMP, 0x7A, amEXT20, bmNone}, // Jump
	{HC16_JSR, 0x89, amIND20_X, bmNone}, // Jump to Subroutine Push (PC)
	{HC16_JSR, 0x99, amIND20_Y, bmNone}, // Jump to Subroutine Push (PC)
	{HC16_JSR, 0xA9, amIND20_Z, bmNone}, // Jump to Subroutine Push (PC)
	{HC16_JSR, 0xFA, amEXT20, bmNone}, // Jump to Subroutine Push (PC)
	{HC16_LBCC, 0x3784, amREL16, bmNone}, // Long Branch if Carry Clear
	{HC16_LBCS, 0x3785, amREL16, bmNone}, // Long Branch if Carry Set
	{HC16_LBEQ, 0x3787, amREL16, bmNone}, // Long Branch if Equal to Zero
	{HC16_LBEV, 0x3791, amREL16, bmNone}, // Long Branch if EV Set 
	{HC16_LBGE, 0x378C, amREL16, bmNone}, // Long Branch if Greater Than or Equal to Zero
	{HC16_LBGT, 0x378E, amREL16, bmNone}, // Long Branch if Greater Than Zero
	{HC16_LBHI, 0x3782, amREL16, bmNone}, // Long Branch if Higher 
	{HC16_LBLE, 0x378F, amREL16, bmNone}, // Long Branch if Less Than or Equal to Zero
	{HC16_LBLS, 0x3783, amREL16, bmNone}, // Long Branch if Lower or Same
	{HC16_LBLT, 0x378D, amREL16, bmNone}, // Long Branch if Less Than Zero
	{HC16_LBMI, 0x378B, amREL16, bmNone}, // Long Branch if Minus 
	{HC16_LBMV, 0x3790, amREL16, bmNone}, // Long Branch if MV Set 
	{HC16_LBNE, 0x3786, amREL16, bmNone}, // Long Branch if Not Equal to Zero
	{HC16_LBPL, 0x378A, amREL16, bmNone}, // Long Branch if Plus 
	{HC16_LBRA, 0x3780, amREL16, bmNone}, // Long Branch Always 
	{HC16_LBRN, 0x3781, amREL16, bmNone}, // Long Branch Never 
	{HC16_LBSR, 0x27F9, amREL16, bmNone}, // Long Branch to Subroutine
	{HC16_LBVC, 0x3788, amREL16, bmNone}, // Long Branch if Overflow Clear
	{HC16_LBVS, 0x3789, amREL16, bmNone}, // Long Branch if Overflow Set
	{HC16_LDAA, 0x1745, amIND16_X, bmNone}, // Load A 
	{HC16_LDAA, 0x1755, amIND16_Y, bmNone}, // Load A 
	{HC16_LDAA, 0x1765, amIND16_Z, bmNone}, // Load A 
	{HC16_LDAA, 0x1775, amEXT, bmNone}, // Load A 
	{HC16_LDAA, 0x2745, amE_X, bmNone}, // Load A 
	{HC16_LDAA, 0x2755, amE_Y, bmNone}, // Load A 
	{HC16_LDAA, 0x2765, amE_Z, bmNone}, // Load A 
	{HC16_LDAA, 0x45, amIND8_X, bmNone}, // Load A 
	{HC16_LDAA, 0x55, amIND8_Y, bmNone}, // Load A 
	{HC16_LDAA, 0x65, amIND8_Z, bmNone}, // Load A 
	{HC16_LDAA, 0x75, amIMM8, bmNone}, // Load A 
	{HC16_LDAB, 0x17C5, amIND16_X, bmNone}, // Load B 
	{HC16_LDAB, 0x17D5, amIND16_Y, bmNone}, // Load B 
	{HC16_LDAB, 0x17E5, amIND16_Z, bmNone}, // Load B 
	{HC16_LDAB, 0x17F5, amEXT, bmNone}, // Load B 
	{HC16_LDAB, 0x27C5, amE_X, bmNone}, // Load B 
	{HC16_LDAB, 0x27D5, amE_Y, bmNone}, // Load B 
	{HC16_LDAB, 0x27E5, amE_Z, bmNone}, // Load B 
	{HC16_LDAB, 0xC5, amIND8_X, bmNone}, // Load B 
	{HC16_LDAB, 0xD5, amIND8_Y, bmNone}, // Load B 
	{HC16_LDAB, 0xE5, amIND8_Z, bmNone}, // Load B 
	{HC16_LDAB, 0xF5, amIMM8, bmNone}, // Load B 
	{HC16_LDD, 0x2785, amE_X, bmNone}, // Load D 
	{HC16_LDD, 0x2795, amE_Y, bmNone}, // Load D 
	{HC16_LDD, 0x27A5, amE_Z, bmNone}, // Load D 
	{HC16_LDD, 0x37B5, amIMM16, bmNone}, // Load D 
	{HC16_LDD, 0x37C5, amIND16_X, bmNone}, // Load D 
	{HC16_LDD, 0x37D5, amIND16_Y, bmNone}, // Load D 
	{HC16_LDD, 0x37E5, amIND16_Z, bmNone}, // Load D 
	{HC16_LDD, 0x37F5, amEXT, bmNone}, // Load D 
	{HC16_LDD, 0x85, amIND8_X, bmNone}, // Load D 
	{HC16_LDD, 0x95, amIND8_Y, bmNone}, // Load D 
	{HC16_LDD, 0xA5, amIND8_Z, bmNone}, // Load D 
	{HC16_LDE, 0x3735, amIMM16, bmNone}, // Load E 
	{HC16_LDE, 0x3745, amIND16_X, bmNone}, // Load E 
	{HC16_LDE, 0x3755, amIND16_Y, bmNone}, // Load E 
	{HC16_LDE, 0x3765, amIND16_Z, bmNone}, // Load E 
	{HC16_LDE, 0x3775, amEXT, bmNone}, // Load E 
	{HC16_LDED, 0x2771, amEXT, bmNone}, // Load Concatenated
	{HC16_LDHI, 0x27B0, amEXT, bmNone}, // Initialize H and I 
	{HC16_LDS, 0x17CF, amIND16_X, bmNone}, // Load SP 
	{HC16_LDS, 0x17DF, amIND16_Y, bmNone}, // Load SP 
	{HC16_LDS, 0x17EF, amIND16_Z, bmNone}, // Load SP 
	{HC16_LDS, 0x17FF, amEXT, bmNone}, // Load SP 
	{HC16_LDS, 0x37BF, amIMM16, bmNone}, // Load SP 
	{HC16_LDS, 0xCF, amIND8_X, bmNone}, // Load SP 
	{HC16_LDS, 0xDF, amIND8_Y, bmNone}, // Load SP 
	{HC16_LDS, 0xEF, amIND8_Z, bmNone}, // Load SP 
	{HC16_LDX, 0x17CC, amIND16_X, bmNone}, // Load IX 
	{HC16_LDX, 0x17DC, amIND16_Y, bmNone}, // Load IX 
	{HC16_LDX, 0x17EC, amIND16_Z, bmNone}, // Load IX 
	{HC16_LDX, 0x17FC, amEXT, bmNone}, // Load IX 
	{HC16_LDX, 0x37BC, amIMM16, bmNone}, // Load IX 
	{HC16_LDX, 0xCC, amIND8_X, bmNone}, // Load IX 
	{HC16_LDX, 0xDC, amIND8_Y, bmNone}, // Load IX 
	{HC16_LDX, 0xEC, amIND8_Z, bmNone}, // Load IX 
	{HC16_LDY, 0x17CD, amIND16_X, bmNone}, // Load IY 
	{HC16_LDY, 0x17DD, amIND16_Y, bmNone}, // Load IY 
	{HC16_LDY, 0x17ED, amIND16_Z, bmNone}, // Load IY 
	{HC16_LDY, 0x17FD, amEXT, bmNone}, // Load IY 
	{HC16_LDY, 0x37BD, amIMM16, bmNone}, // Load IY 
	{HC16_LDY, 0xCD, amIND8_X, bmNone}, // Load IY 
	{HC16_LDY, 0xDD, amIND8_Y, bmNone}, // Load IY 
	{HC16_LDY, 0xED, amIND8_Z, bmNone}, // Load IY 
	{HC16_LDZ, 0x17CE, amIND16_X, bmNone}, // Load IZ 
	{HC16_LDZ, 0x17DE, amIND16_Y, bmNone}, // Load IZ 
	{HC16_LDZ, 0x17EE, amIND16_Z, bmNone}, // Load IZ 
	{HC16_LDZ, 0x17FE, amEXT, bmNone}, // Load IZ 
	{HC16_LDZ, 0x37BE, amIMM16, bmNone}, // Load IZ 
	{HC16_LDZ, 0xCE, amIND8_X, bmNone}, // Load IZ 
	{HC16_LDZ, 0xDE, amIND8_Y, bmNone}, // Load IZ 
	{HC16_LDZ, 0xEE, amIND8_Z, bmNone}, // Load IZ 
	{HC16_LPSTOP, 0x27F1, amINH, bmNone}, // Low Power Stop If S
	{HC16_LSR, 0x0F, amIND8_X, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x170F, amIND16_X, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x171F, amIND16_Y, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x172F, amIND16_Z, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x173F, amEXT, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x1F, amIND8_Y, bmNone}, // Logical Shift Right 
	{HC16_LSR, 0x2F, amIND8_Z, bmNone}, // Logical Shift Right 
	{HC16_LSRA, 0x370F, amINH, bmNone}, // Logical Shift Right A 
	{HC16_LSRB, 0x371F, amINH, bmNone}, // Logical Shift Right B 
	{HC16_LSRD, 0x27FF, amINH, bmNone}, // Logical Shift Right D 
	{HC16_LSRE, 0x277F, amINH, bmNone}, // Logical Shift Right E 
	{HC16_LSRW, 0x270F, amIND16_X, bmNone}, // Logical Shift Right Word
	{HC16_LSRW, 0x271F, amIND16_Y, bmNone}, // Logical Shift Right Word
	{HC16_LSRW, 0x272F, amIND16_Z, bmNone}, // Logical Shift Right Word
	{HC16_LSRW, 0x273F, amEXT, bmNone}, // Logical Shift Right Word
	{HC16_MAC, 0x7B, amIMM8, bmNone}, // Multiply and Accumulate Signed 16-Bit Fractions
	{HC16_MOVB, 0x30, amIXP_EXT, bmNone}, // Move Byte (
	{HC16_MOVB, 0x32, amEXT_IXP, bmNone}, // Move Byte (
	{HC16_MOVB, 0x37FE, amEXT_EXT, bmNone}, // Move Byte (
	{HC16_MOVW, 0x31, amIXP_EXT, bmNone}, // Move Word 
	{HC16_MOVW, 0x33, amEXT_IXP, bmNone}, // Move Word 
	{HC16_MOVW, 0x37FF, amEXT_EXT, bmNone}, // Move Word 
	{HC16_MUL, 0x3724, amINH, bmNone}, // Multiply 
	{HC16_NEG, 0x12, amIND8_Y, bmNone}, // Negate Memory 
	{HC16_NEG, 0x1702, amIND16_X, bmNone}, // Negate Memory 
	{HC16_NEG, 0x1712, amIND16_Y, bmNone}, // Negate Memory 
	{HC16_NEG, 0x1722, amIND16_Z, bmNone}, // Negate Memory 
	{HC16_NEG, 0x1732, amEXT, bmNone}, // Negate Memory 
	{HC16_NEG, 0x2, amIND8_X, bmNone}, // Negate Memory 
	{HC16_NEG, 0x22, amIND8_Z, bmNone}, // Negate Memory 
	{HC16_NEGA, 0x3702, amINH, bmNone}, // Negate A 
	{HC16_NEGB, 0x3712, amINH, bmNone}, // Negate B 
	{HC16_NEGD, 0x27F2, amINH, bmNone}, // Negate D 
	{HC16_NEGE, 0x2772, amINH, bmNone}, // Negate E 
	{HC16_NEGW, 0x2702, amIND16_X, bmNone}, // Negate Memory Word 
	{HC16_NEGW, 0x2712, amIND16_Y, bmNone}, // Negate Memory Word 
	{HC16_NEGW, 0x2722, amIND16_Z, bmNone}, // Negate Memory Word 
	{HC16_NEGW, 0x2732, amEXT, bmNone}, // Negate Memory Word 
	{HC16_NOP, 0x274C, amINH, bmNone}, // Null Operation 
	{HC16_ORAA, 0x1747, amIND16_X, bmNone}, // OR A
	{HC16_ORAA, 0x1757, amIND16_Y, bmNone}, // OR A
	{HC16_ORAA, 0x1767, amIND16_Z, bmNone}, // OR A
	{HC16_ORAA, 0x1777, amEXT, bmNone}, // OR A
	{HC16_ORAA, 0x2747, amE_X, bmNone}, // OR A
	{HC16_ORAA, 0x2757, amE_Y, bmNone}, // OR A
	{HC16_ORAA, 0x2767, amE_Z, bmNone}, // OR A
	{HC16_ORAA, 0x47, amIND8_X, bmNone}, // OR A
	{HC16_ORAA, 0x57, amIND8_Y, bmNone}, // OR A
	{HC16_ORAA, 0x67, amIND8_Z, bmNone}, // OR A
	{HC16_ORAA, 0x77, amIMM8, bmNone}, // OR A
	{HC16_ORAB, 0x17C7, amIND16_X, bmNone}, // OR B 
	{HC16_ORAB, 0x17D7, amIND16_Y, bmNone}, // OR B 
	{HC16_ORAB, 0x17E7, amIND16_Z, bmNone}, // OR B 
	{HC16_ORAB, 0x17F7, amEXT, bmNone}, // OR B 
	{HC16_ORAB, 0x27C7, amE_X, bmNone}, // OR B 
	{HC16_ORAB, 0x27D7, amE_Y, bmNone}, // OR B 
	{HC16_ORAB, 0x27E7, amE_Z, bmNone}, // OR B 
	{HC16_ORAB, 0xC7, amIND8_X, bmNone}, // OR B 
	{HC16_ORAB, 0xD7, amIND8_Y, bmNone}, // OR B 
	{HC16_ORAB, 0xE7, amIND8_Z, bmNone}, // OR B 
	{HC16_ORAB, 0xF7, amIMM8, bmNone}, // OR B 
	{HC16_ORD, 0x2787, amE_X, bmNone}, // OR D 
	{HC16_ORD, 0x2797, amE_Y, bmNone}, // OR D 
	{HC16_ORD, 0x27A7, amE_Z, bmNone}, // OR D 
	{HC16_ORD, 0x37B7, amIMM16, bmNone}, // OR D 
	{HC16_ORD, 0x37C7, amIND16_X, bmNone}, // OR D 
	{HC16_ORD, 0x37D7, amIND16_Y, bmNone}, // OR D 
	{HC16_ORD, 0x37E7, amIND16_Z, bmNone}, // OR D 
	{HC16_ORD, 0x37F7, amEXT, bmNone}, // OR D 
	{HC16_ORD, 0x87, amIND8_X, bmNone}, // OR D 
	{HC16_ORD, 0x97, amIND8_Y, bmNone}, // OR D 
	{HC16_ORD, 0xA7, amIND8_Z, bmNone}, // OR D 
	{HC16_ORE, 0x3737, amIMM16, bmNone}, // OR E 
	{HC16_ORE, 0x3747, amIND16_X, bmNone}, // OR E 
	{HC16_ORE, 0x3757, amIND16_Y, bmNone}, // OR E 
	{HC16_ORE, 0x3767, amIND16_Z, bmNone}, // OR E 
	{HC16_ORE, 0x3777, amEXT, bmNone}, // OR E 
	{HC16_ORP, 0x373B, amIMM16, bmNone}, // 1 OR Condition Code Register
	{HC16_PSHA, 0x3708, amINH, bmNone}, // Push A
	{HC16_PSHB, 0x3718, amINH, bmNone}, // Push B 
	{HC16_PSHM, 0x34, amIMM8, bmReg}, // Push Multiple Registers
	{HC16_PSHMAC, 0x27B8, amINH, bmNone}, // Push MAC Registers 
	{HC16_PULA, 0x3709, amINH, bmNone}, // Pull A 
	{HC16_PULB, 0x3719, amINH, bmNone}, // Pull B 
	{HC16_PULM, 0x35, amIMM8, bmReg}, 
	{HC16_PULMAC, 0x27B9, amINH, bmNone}, // Pull MAC State 
	{HC16_RMAC, 0xFB, amIMM8, bmNone}, // Repeating Multiply and Accumulate Signed 16-Bit Fractions
	{HC16_ROL, 0x0C, amIND8_X, bmNone}, // Rotate Left 
	{HC16_ROL, 0x170C, amIND16_X, bmNone}, // Rotate Left 
	{HC16_ROL, 0x171C, amIND16_Y, bmNone}, // Rotate Left 
	{HC16_ROL, 0x172C, amIND16_Z, bmNone}, // Rotate Left 
	{HC16_ROL, 0x173C, amEXT, bmNone}, // Rotate Left 
	{HC16_ROL, 0x1C, amIND8_Y, bmNone}, // Rotate Left 
	{HC16_ROL, 0x2C, amIND8_Z, bmNone}, // Rotate Left 
	{HC16_ROLA, 0x370C, amINH, bmNone}, // Rotate Left A 
	{HC16_ROLB, 0x371C, amINH, bmNone}, // Rotate Left B 
	{HC16_ROLD, 0x27FC, amINH, bmNone}, // Rotate Left D 
	{HC16_ROLE, 0x277C, amINH, bmNone}, // Rotate Left E 
	{HC16_ROLW, 0x270C, amIND16_X, bmNone}, // Rotate Left Word 
	{HC16_ROLW, 0x271C, amIND16_Y, bmNone}, // Rotate Left Word 
	{HC16_ROLW, 0x272C, amIND16_Z, bmNone}, // Rotate Left Word 
	{HC16_ROLW, 0x273C, amEXT, bmNone}, // Rotate Left Word 
	{HC16_ROR, 0x0E, amIND8_X, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x170E, amIND16_X, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x171E, amIND16_Y, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x172E, amIND16_Z, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x173E, amEXT, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x1E, amIND8_Y, bmNone}, // Rotate Right Byte 
	{HC16_ROR, 0x2E, amIND8_Z, bmNone}, // Rotate Right Byte 
	{HC16_RORA, 0x370E, amINH, bmNone}, // Rotate Right A 
	{HC16_RORB, 0x371E, amINH, bmNone}, // Rotate Right B 
	{HC16_RORD, 0x27FE, amINH, bmNone}, // Rotate Right D
	{HC16_RORE, 0x277E, amINH, bmNone}, // Rotate Right E 
	{HC16_RORW, 0x270E, amIND16_X, bmNone}, // Rotate Right Word 
	{HC16_RORW, 0x271E, amIND16_Y, bmNone}, // Rotate Right Word 
	{HC16_RORW, 0x272E, amIND16_Z, bmNone}, // Rotate Right Word 
	{HC16_RORW, 0x273E, amEXT, bmNone}, // Rotate Right Word 
	{HC16_RTI, 0x2777, amINH, bmNone}, 
	{HC16_RTS, 0x27F7, amINH, bmNone}, 
	{HC16_SBA, 0x370A, amINH, bmNone}, // Subtract B from A 
	{HC16_SBCA, 0x1742, amIND16_X, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x1752, amIND16_Y, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x1762, amIND16_Z, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x1772, amEXT, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x2742, amE_X, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x2752, amE_Y, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x2762, amE_Z, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x42, amIND8_X, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x52, amIND8_Y, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x62, amIND8_Z, bmNone}, // Subtract with Carry from A
	{HC16_SBCA, 0x72, amIMM8, bmNone}, // Subtract with Carry from A
	{HC16_SBCB, 0x17C2, amIND16_X, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x17D2, amIND16_Y, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x17E2, amIND16_Z, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x17F2, amEXT, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x27C2, amE_X, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x27D2, amE_Y, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0x27E2, amE_Z, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0xC2, amIND8_X, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0xD2, amIND8_Y, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0xE2, amIND8_Z, bmNone}, // Subtract with Carry from B
	{HC16_SBCB, 0xF2, amIMM8, bmNone}, // Subtract with Carry from B
	{HC16_SBCD, 0x2782, amE_X, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x2792, amE_Y, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x27A2, amE_Z, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x37B2, amIMM16, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x37C2, amIND16_X, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x37D2, amIND16_Y, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x37E2, amIND16_Z, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x37F2, amEXT, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x82, amIND8_X, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0x92, amIND8_Y, bmNone}, // Subtract with Carry from D
	{HC16_SBCD, 0xA2, amIND8_Z, bmNone}, // Subtract with Carry from D
	{HC16_SBCE, 0x3732, amIMM16, bmNone}, // Subtract with Carry from E
	{HC16_SBCE, 0x3742, amIND16_X, bmNone}, // Subtract with Carry from E
	{HC16_SBCE, 0x3752, amIND16_Y, bmNone}, // Subtract with Carry from E
	{HC16_SBCE, 0x3762, amIND16_Z, bmNone}, // Subtract with Carry from E
	{HC16_SBCE, 0x3772, amEXT, bmNone}, // Subtract with Carry from E
	{HC16_SDE, 0x2779, amINH, bmNone}, // Subtract D from E 
	{HC16_STAA, 0x174A, amIND16_X, bmNone}, // Store A 
	{HC16_STAA, 0x175A, amIND16_Y, bmNone}, // Store A 
	{HC16_STAA, 0x176A, amIND16_Z, bmNone}, // Store A 
	{HC16_STAA, 0x177A, amEXT, bmNone}, // Store A 
	{HC16_STAA, 0x274A, amE_X, bmNone}, // Store A 
	{HC16_STAA, 0x275A, amE_Y, bmNone}, // Store A 
	{HC16_STAA, 0x276A, amE_Z, bmNone}, // Store A 
	{HC16_STAA, 0x4A, amIND8_X, bmNone}, // Store A 
	{HC16_STAA, 0x5A, amIND8_Y, bmNone}, // Store A 
	{HC16_STAA, 0x6A, amIND8_Z, bmNone}, // Store A 
	{HC16_STAB, 0x17CA, amIND16_X, bmNone}, // Store B 
	{HC16_STAB, 0x17DA, amIND16_Y, bmNone}, // Store B 
	{HC16_STAB, 0x17EA, amIND16_Z, bmNone}, // Store B 
	{HC16_STAB, 0x17FA, amEXT, bmNone}, // Store B 
	{HC16_STAB, 0x27CA, amE_X, bmNone}, // Store B 
	{HC16_STAB, 0x27DA, amE_Y, bmNone}, // Store B 
	{HC16_STAB, 0x27EA, amE_Z, bmNone}, // Store B 
	{HC16_STAB, 0xCA, amIND8_X, bmNone}, // Store B 
	{HC16_STAB, 0xDA, amIND8_Y, bmNone}, // Store B 
	{HC16_STAB, 0xEA, amIND8_Z, bmNone}, // Store B 
	{HC16_STD, 0x278A, amE_X, bmNone}, // Store D 
	{HC16_STD, 0x279A, amE_Y, bmNone}, // Store D 
	{HC16_STD, 0x27AA, amE_Z, bmNone}, // Store D 
	{HC16_STD, 0x37CA, amIND16_X, bmNone}, // Store D 
	{HC16_STD, 0x37DA, amIND16_Y, bmNone}, // Store D 
	{HC16_STD, 0x37EA, amIND16_Z, bmNone}, // Store D 
	{HC16_STD, 0x37FA, amEXT, bmNone}, // Store D 
	{HC16_STD, 0x8A, amIND8_X, bmNone}, // Store D 
	{HC16_STD, 0x9A, amIND8_Y, bmNone}, // Store D 
	{HC16_STD, 0xAA, amIND8_Z, bmNone}, // Store D 
	{HC16_STE, 0x374A, amIND16_X, bmNone}, // Store E 
	{HC16_STE, 0x375A, amIND16_Y, bmNone}, // Store E 
	{HC16_STE, 0x376A, amIND16_Z, bmNone}, // Store E 
	{HC16_STE, 0x377A, amEXT, bmNone}, // Store E 
	{HC16_STED, 0x2773, amEXT, bmNone}, // Store Concatenated D and E
	{HC16_STS, 0x178F, amIND16_X, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0x179F, amIND16_Y, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0x17AF, amIND16_Z, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0x17BF, amEXT, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0x8F, amIND8_X, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0x9F, amIND8_Y, bmNone}, // Store Stack Pointer 
	{HC16_STS, 0xAF, amIND8_Z, bmNone}, // Store Stack Pointer 
	{HC16_STX, 0x178C, amIND16_X, bmNone}, // Store IX 
	{HC16_STX, 0x179C, amIND16_Y, bmNone}, // Store IX 
	{HC16_STX, 0x17AC, amIND16_Z, bmNone}, // Store IX 
	{HC16_STX, 0x17BC, amEXT, bmNone}, // Store IX 
	{HC16_STX, 0x8C, amIND8_X, bmNone}, // Store IX 
	{HC16_STX, 0x9C, amIND8_Y, bmNone}, // Store IX 
	{HC16_STX, 0xAC, amIND8_Z, bmNone}, // Store IX 
	{HC16_STY, 0x178D, amIND16_X, bmNone}, // Store IY 
	{HC16_STY, 0x179D, amIND16_Y, bmNone}, // Store IY 
	{HC16_STY, 0x17AD, amIND16_Z, bmNone}, // Store IY 
	{HC16_STY, 0x17BD, amEXT, bmNone}, // Store IY 
	{HC16_STY, 0x8D, amIND8_X, bmNone}, // Store IY 
	{HC16_STY, 0x9D, amIND8_Y, bmNone}, // Store IY 
	{HC16_STY, 0xAD, amIND8_Z, bmNone}, // Store IY 
	{HC16_STZ, 0x178E, amIND16_X, bmNone}, // Store Z 
	{HC16_STZ, 0x179E, amIND16_Y, bmNone}, // Store Z 
	{HC16_STZ, 0x17AE, amIND16_Z, bmNone}, // Store Z 
	{HC16_STZ, 0x17BE, amEXT, bmNone}, // Store Z 
	{HC16_STZ, 0x8E, amIND8_X, bmNone}, // Store Z 
	{HC16_STZ, 0x9E, amIND8_Y, bmNone}, // Store Z 
	{HC16_STZ, 0xAE, amIND8_Z, bmNone}, // Store Z 
	{HC16_SUBA, 0x1740, amIND16_X, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x1750, amIND16_Y, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x1760, amIND16_Z, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x1770, amEXT, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x2740, amE_X, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x2750, amE_Y, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x2760, amE_Z, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x40, amIND8_X, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x50, amIND8_Y, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x60, amIND8_Z, bmNone}, // Subtract from A 
	{HC16_SUBA, 0x70, amIMM8, bmNone}, // Subtract from A 
	{HC16_SUBB, 0x17C0, amIND16_X, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x17D0, amIND16_Y, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x17E0, amIND16_Z, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x17F0, amEXT, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x27C0, amE_X, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x27D0, amE_Y, bmNone}, // Subtract from B 
	{HC16_SUBB, 0x27E0, amE_Z, bmNone}, // Subtract from B 
	{HC16_SUBB, 0xC0, amIND8_X, bmNone}, // Subtract from B 
	{HC16_SUBB, 0xD0, amIND8_Y, bmNone}, // Subtract from B 
	{HC16_SUBB, 0xE0, amIND8_Z, bmNone}, // Subtract from B 
	{HC16_SUBB, 0xF0, amIMM8, bmNone}, // Subtract from B 
	{HC16_SUBD, 0x2780, amE_X, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x2790, amE_Y, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x27A0, amE_Z, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x37B0, amIMM16, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x37C0, amIND16_X, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x37D0, amIND16_Y, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x37E0, amIND16_Z, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x37F0, amEXT, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x80, amIND8_X, bmNone}, // Subtract from D 
	{HC16_SUBD, 0x90, amIND8_Y, bmNone}, // Subtract from D 
	{HC16_SUBD, 0xA0, amIND8_Z, bmNone}, // Subtract from D 
	{HC16_SUBE, 0x3730, amIMM16, bmNone}, // Subtract from E 
	{HC16_SUBE, 0x3740, amIND16_X, bmNone}, // Subtract from E 
	{HC16_SUBE, 0x3750, amIND16_Y, bmNone}, // Subtract from E 
	{HC16_SUBE, 0x3760, amIND16_Z, bmNone}, // Subtract from E 
	{HC16_SUBE, 0x3770, amEXT, bmNone}, // Subtract from E 
	{HC16_SWI, 0x3720, amINH, bmNone}, // Software Interrupt 
	{HC16_SXT, 0x27F8, amINH, bmNone}, // Sign Extend B into A
	{HC16_TAB, 0x3717, amINH, bmNone}, // Transfer A to B 
	{HC16_TAP, 0x37FD, amINH, bmNone}, // Transfer A to CCR 
	{HC16_TBA, 0x3707, amINH, bmNone}, // Transfer B to A
	{HC16_TBEK, 0x27FA, amINH, bmNone}, // Transfer B to EK 
	{HC16_TBSK, 0x379F, amINH, bmNone}, // Transfer B to SK 
	{HC16_TBXK, 0x379C, amINH, bmNone}, // Transfer B to XK 
	{HC16_TBYK, 0x379D, amINH, bmNone}, // Transfer B to YK 
	{HC16_TBZK, 0x379E, amINH, bmNone}, // Transfer B to ZK 
	{HC16_TDE, 0x277B, amINH, bmNone}, // Transfer D to E 
	{HC16_TDMSK, 0x372F, amINH, bmNone}, // Transfer D to XMSK : YMSK
	{HC16_TDP, 0x372D, amINH, bmNone}, 
	{HC16_TED, 0x27FB, amINH, bmNone}, // Transfer E to D 
	{HC16_TEDM, 0x27B1, amINH, bmNone}, // Transfer E and D to AM[31:0] Sign Extend AM
	{HC16_TEKB, 0x27BB, amINH, bmNone}, // Transfer EK to B 
	{HC16_TEM, 0x27B2, amINH, bmNone}, // Transfer E to AM[31:16] Sign Extend AM Clear AM LSB
	{HC16_TMER, 0x27B4, amINH, bmNone}, // Transfer Rounded AM to E
	{HC16_TMET, 0x27B5, amINH, bmNone}, // Transfer Truncated AM to E
	{HC16_TMXED, 0x27B3, amINH, bmNone}, // Transfer AM to
	{HC16_TPA, 0x37FC, amINH, bmNone}, // Transfer CCR to A 
	{HC16_TPD, 0x372C, amINH, bmNone}, // Transfer CCR to D 
	{HC16_TSKB, 0x37AF, amINH, bmNone}, // Transfer SK to B 
	{HC16_TST, 0x16, amIND8_Y, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x1706, amIND16_X, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x1716, amIND16_Y, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x1726, amIND16_Z, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x1736, amEXT, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x26, amIND8_Z, bmNone}, // Test Byte Zero or Minus
	{HC16_TST, 0x6, amIND8_X, bmNone}, // Test Byte Zero or Minus
	{HC16_TSTA, 0x3706, amINH, bmNone}, // Test A for Zero or Minus
	{HC16_TSTB, 0x3716, amINH, bmNone}, // Test B for Zero or Minus
	{HC16_TSTD, 0x27F6, amINH, bmNone}, // Test D for Zero or Minus
	{HC16_TSTE, 0x2776, amINH, bmNone}, // Test E for Zero or Minus
	{HC16_TSTW, 0x2706, amIND16_X, bmNone}, // Test for Zero or Minus Word
	{HC16_TSTW, 0x2716, amIND16_Y, bmNone}, // Test for Zero or Minus Word
	{HC16_TSTW, 0x2726, amIND16_Z, bmNone}, // Test for Zero or Minus Word
	{HC16_TSTW, 0x2736, amEXT, bmNone}, // Test for Zero or Minus Word
	{HC16_TSX, 0x274F, amINH, bmNone}, // Transfer SP to X 
	{HC16_TSY, 0x275F, amINH, bmNone}, // Transfer SP to Y 
	{HC16_TSZ, 0x276F, amINH, bmNone}, // Transfer SP to Z 
	{HC16_TXKB, 0x37AC, amINH, bmNone}, // Transfer XK to B 
	{HC16_TXS, 0x374E, amINH, bmNone}, // Transfer X to SP 
	{HC16_TXY, 0x275C, amINH, bmNone}, // Transfer X to Y 
	{HC16_TXZ, 0x276C, amINH, bmNone}, // Transfer X to Z 
	{HC16_TYKB, 0x37AD, amINH, bmNone}, // Transfer YK to B 
	{HC16_TYS, 0x375E, amINH, bmNone}, // Transfer Y to SP 
	{HC16_TYX, 0x274D, amINH, bmNone}, // Transfer Y to X 
	{HC16_TYZ, 0x276D, amINH, bmNone}, // Transfer Y to Z 
	{HC16_TZKB, 0x37AE, amINH, bmNone}, // Transfer ZK to B 
	{HC16_TZS, 0x376E, amINH, bmNone}, // Transfer Z to SP 
	{HC16_TZX, 0x274E, amINH, bmNone}, // Transfer Z to X 
	{HC16_TZY, 0x275E, amINH, bmNone}, // Transfer Z to Y
	{HC16_WAI, 0x27F3, amINH, bmNone}, // Wait for Interrupt 
	{HC16_XGAB, 0x371A, amINH, bmNone}, // Exchange A with B 
	{HC16_XGDE, 0x277A, amINH, bmNone}, // Exchange D with E 
	{HC16_XGDX, 0x37CC, amINH, bmNone}, // Exchange D with IX 
	{HC16_XGDY, 0x37DC, amINH, bmNone}, // Exchange D with IY 
	{HC16_XGDZ, 0x37EC, amINH, bmNone}, // Exchange D with IZ 
	{HC16_XGEX, 0x374C, amINH, bmNone}, // Exchange E with IX 
	{HC16_XGEY, 0x375C, amINH, bmNone}, // Exchange E with IY 
	{HC16_XGEZ, 0x376C, amINH, bmNone}, // Exchange E with IZ
	{HC16_last, 0     , 0    , 0},
};
