#ifndef __INSTRS_HPP
#define __INSTRS_HPP

struct instruc_decode_t
{
	int instrnum;	// enum'd instruction
	short opcode;	// 1-2 byte opcode
	int addrmode;	// addressing mode
	int bitmode;	// bit mask (if any)
};

extern instruc_t Instructions[];
extern instruc_decode_t InstrDecoder[];

enum nameNum {
HC16_null = 0,   // Unknown Operation

HC16_ABA,	// Add B to A 
HC16_ABX,	// Add B to IX 
HC16_ABY,	// Add B to IY 
HC16_ABZ,	// Add B to IZ
HC16_ACE,	// Add E to AM
HC16_ACED,	// Add E : D to AM 
HC16_ADCA,	// Add with Carry to A
HC16_ADCB,	// Add with Carry to B 
HC16_ADCD,	// Add with Carry to D 
HC16_ADCE,	// Add with Carry to E 
HC16_ADDA,	// Add to A 
HC16_ADDB,	// Add to B 
HC16_ADDD,	// Add to D 
HC16_ADDE,	// Add to E 
HC16_ADE,	// Add D to E
HC16_ADX,	// Add D to IX 
HC16_ADY,	// Add D to IY 
HC16_ADZ,	// Add D to IZ 
HC16_AEX,	// Add E to IX 
HC16_AEY,	// Add E to IY 
HC16_AEZ,	// Add E to IZ 
HC16_AIS,	// Add Immediate Data to Stack Pointer
HC16_AIX,	// Add Immediate Value to IX
HC16_AIY,	// Add Immediate Value to IY
HC16_AIZ,	// Add Immediate Value to IZ
HC16_ANDA,	// AND A 
HC16_ANDB,	// AND B 
HC16_ANDD,	// AND D 
HC16_ANDE,	// AND E 
HC16_ANDP,	// AND CCR 
HC16_ASL,	// Arithmetic Shift Left 
HC16_ASLA,	// Arithmetic Shift Left A 
HC16_ASLB,	// Arithmetic Shift Left B 
HC16_ASLD,	// Arithmetic Shift Left D 
HC16_ASLE,	// Arithmetic Shift Left E 
HC16_ASLM,	// Arithmetic Shift Left
HC16_ASLW,	// Arithmetic Shift Left Word
HC16_ASR,	// Arithmetic Shift Right 
HC16_ASRA,	// Arithmetic Shift Right A
HC16_ASRB,	// Arithmetic Shift Right B
HC16_ASRD,	// Arithmetic Shift Right D
HC16_ASRE,	// Arithmetic Shift Right E
HC16_ASRM,	// Arithmetic Shift Right AM
HC16_ASRW,	// Arithmetic Shift Right Word
HC16_BCC,	// Branch if Carry Clear 
HC16_BCLR,	// Clear Bit(s)
HC16_BCLRW,	// Clear Bit(s) in a Word
HC16_BCS,	// Branch if Carry Set 
HC16_BEQ,	// Branch if Equal
HC16_BGE,	// Branch if Greater Than or Equal to Zero
HC16_BGND,	// Enter Background Debug Mode If BDM enabled, begin debug; else, illegal instruction trap
HC16_BGT,	// Branch if Greater Than Zero
HC16_BHI,	// Branch if Higher 
HC16_BITA,	// Bit Test A 
HC16_BITB,	// Bit Test B 
HC16_BLE,	// Branch if Less Than or Equal to Zero
HC16_BLS,	// Branch if Lower or Same
HC16_BLT,	// Branch if Less Than Zero
HC16_BMI,	// Branch if Minus 
HC16_BNE,	// Branch if Not Equal 
HC16_BPL,	// Branch if Plus 
HC16_BRA,	// Branch Always 
HC16_BRCLR,// Branch if Bit(s) Clear
HC16_BRN,	// Branch Never 
HC16_BRSET,// Branch if Bit(s) Set
HC16_BSET,	// Set Bit(s) 
HC16_BSETW,	// Set Bit(s) in Word 
HC16_BSR,	// Branch to Subroutine 
HC16_BVC,	// Branch if Overflow Clear
HC16_BVS,	// Branch if Overflow Set
HC16_CBA,	// Compare A to B 
HC16_CLR,	// Clear a Byte in Memory
HC16_CLRA,	// Clear A 
HC16_CLRB,	// Clear B 
HC16_CLRD,	// Clear D 
HC16_CLRE,	// Clear E
HC16_CLRM,	// Clear AM
HC16_CLRW,	// Clear a Word in Memory
HC16_CMPA,	// Compare A to Memory 
HC16_CMPB,	// Compare B to Memory 
HC16_COM,	// Ones Complement
HC16_COMA,	// Ones Complement A
HC16_COMB,	// Ones Complement B
HC16_COMD,	// Ones Complement D 
HC16_COME,	// Ones Complement E
HC16_COMW,	// Ones Complement Word
HC16_CPD,	// Compare D to Memory 
HC16_CPE,	// Compare E to Memory 
HC16_CPS,	// Compare Stack Pointer to Memory
HC16_CPX,	// Compare IX to Memory
HC16_CPY,	// Compare IY to Memory
HC16_CPZ,	// Compare IZ to Memory
HC16_DAA,	// Decimal Adjust A 
HC16_DEC,	// Decrement Memory 
HC16_DECA,	// Decrement A 
HC16_DECB,	// Decrement B 
HC16_DECW,	// Decrement Memory Word
HC16_EDIV,	// Extended Unsigned Integer Divide
HC16_EDIVS,	// Extended Signed Integer Divide
HC16_EMUL,	// Extended Unsigned Multiply
HC16_EMULS,	// Extended Signed Multiply
HC16_EORA,	// Exclusive OR A
HC16_EORB,	// Exclusive OR B 
HC16_EORD,	// Exclusive OR D 
HC16_EORE,	// Exclusive OR E 
HC16_FDIV,	// Fractional Unsigned Divide
HC16_FMULS,	// Fractional Signed Multiply
HC16_IDIV,	// Integer Divide 
HC16_INC,	// Increment Memory 
HC16_INCA,	// Increment A
HC16_INCB,	// Increment B 
HC16_INCW,	// Increment Memory Word
HC16_JMP,	// Jump ea  PK : PC EXT20
HC16_JSR,	// Jump to Subroutine Push (PC)
HC16_LBCC,	// Long Branch if Carry Clear
HC16_LBCS,	// Long Branch if Carry Set
HC16_LBEQ,	// Long Branch if Equal to Zero
HC16_LBEV,	// Long Branch if EV Set 
HC16_LBGE,	// Long Branch if Greater Than or Equal to Zero
HC16_LBGT,	// Long Branch if Greater Than Zero
HC16_LBHI,	// Long Branch if Higher 
HC16_LBLE,	// Long Branch if Less Than or Equal to Zero
HC16_LBLS,	// Long Branch if Lower or Same
HC16_LBLT,	// Long Branch if Less Than Zero
HC16_LBMI,	// Long Branch if Minus 
HC16_LBMV,	// Long Branch if MV Set 
HC16_LBNE,	// Long Branch if Not Equal to Zero
HC16_LBPL,	// Long Branch if Plus 
HC16_LBRA,	// Long Branch Always 
HC16_LBRN,	// Long Branch Never 
HC16_LBSR,	// Long Branch to Subroutine
HC16_LBVC,	// Long Branch if Overflow Clear
HC16_LBVS,	// Long Branch if Overflow Set
HC16_LDAA,	// Load A 
HC16_LDAB,	// Load B 
HC16_LDD,	// Load D 
HC16_LDE,	// Load E 
HC16_LDED,	// Load Concatenated
HC16_LDHI,	// Initialize H and I 
HC16_LDS,	// Load SP 
HC16_LDX,	// Load IX 
HC16_LDY,	// Load IY 
HC16_LDZ,	// Load IZ 
HC16_LPSTOP,// Low Power Stop If S
HC16_LSR,	// Logical Shift Right 
HC16_LSRA,	// Logical Shift Right A 
HC16_LSRB,	// Logical Shift Right B 
HC16_LSRD,	// Logical Shift Right D 
HC16_LSRE,	// Logical Shift Right E 
HC16_LSRW,	// Logical Shift Right Word
HC16_MAC,	// Multiply and Accumulate Signed 16-Bit Fractions
HC16_MOVB,	// Move Byte (
HC16_MOVW,	// Move Word 
HC16_MUL,	// Multiply 
HC16_NEG,	// Negate Memory 
HC16_NEGA,	// Negate A 
HC16_NEGB,	// Negate B 
HC16_NEGD,	// Negate D 
HC16_NEGE,	// Negate E 
HC16_NEGW,	// Negate Memory Word 
HC16_NOP,	// Null Operation 
HC16_ORAA,	// OR A
HC16_ORAB,	// OR B 
HC16_ORD,	// OR D 
HC16_ORE,	// OR E 
HC16_ORP,	// 1 OR Condition Code Register
HC16_PSHA,	// Push A P
HC16_PSHB,	// Push B 
HC16_PSHM,	// Push Multiple Registers
HC16_PSHMAC,// Push MAC Registers 
HC16_PULA,	// Pull A 
HC16_PULB,	// Pull B 
HC16_PULM,	// Pull Multiple Registers
HC16_PULMAC,// Pull MAC State 
HC16_RMAC,	// Repeating Multiply and Accumulate Signed 16-Bit Fractions
HC16_ROL,	// Rotate Left 
HC16_ROLA,	// Rotate Left A 
HC16_ROLB,	// Rotate Left B 
HC16_ROLD,	// Rotate Left D 
HC16_ROLE,	// Rotate Left E 
HC16_ROLW,	// Rotate Left Word 
HC16_ROR,	// Rotate Right Byte 
HC16_RORA,	// Rotate Right A 
HC16_RORB,	// Rotate Right B 
HC16_RORD,	// Rotate Right D
HC16_RORE,	// Rotate Right E 
HC16_RORW,	// Rotate Right Word 
HC16_RTI,	// Return from Interrupt 
HC16_RTS,	// Return from Subroutine
HC16_SBA,	// Subtract B from A 
HC16_SBCA,	// Subtract with Carry
HC16_SBCB,	// Subtract with Carry from B
HC16_SBCD,	// Subtract with Carry from D
HC16_SBCE,	// Subtract with Carry from E
HC16_SDE,	// Subtract D from E 
HC16_STAA,	// Store A 
HC16_STAB,	// Store B 
HC16_STD,	// Store D 
HC16_STE,	// Store E 
HC16_STED,	// Store Concatenated D and E
HC16_STS,	// Store Stack Pointer 
HC16_STX,	// Store IX 
HC16_STY,	// Store IY 
HC16_STZ,	// Store Z 
HC16_SUBA,	// Subtract from A 
HC16_SUBB,	// Subtract from B 
HC16_SUBD,	// Subtract from D 
HC16_SUBE,	// Subtract from E 
HC16_SWI,	// Software Interrupt 
HC16_SXT,	// Sign Extend B into A If B7 = 1
HC16_TAB,	// Transfer A to B 
HC16_TAP,	// Transfer A to CCR 
HC16_TBA,	// Transfer B to A
HC16_TBEK,	// Transfer B to EK 
HC16_TBSK,	// Transfer B to SK 
HC16_TBXK,	// Transfer B to XK 
HC16_TBYK,	// Transfer B to YK 
HC16_TBZK,	// Transfer B to ZK 
HC16_TDE,	// Transfer D to E 
HC16_TDMSK,	// Transfer D to XMSK : YMSK
HC16_TDP,	// Transfer D to CCR 
HC16_TED,	// Transfer E to D 
HC16_TEDM,	// Transfer E and D to AM[31:0] Sign Extend AM
HC16_TEKB,	// Transfer EK to B 
HC16_TEM,	// Transfer E to AM[31:16] Sign Extend AM Clear AM LSB
HC16_TMER,	// Transfer Rounded AM to E
HC16_TMET,	// Transfer Truncated AM to E
HC16_TMXED,	// Transfer AM to
HC16_TPA,	// Transfer CCR to A 
HC16_TPD,	// Transfer CCR to D 
HC16_TSKB,	// Transfer SK to B 
HC16_TST,	// Test Byte Zero or Minus
HC16_TSTA,	// Test A for Zero or Minus
HC16_TSTB,	// Test B for Zero or Minus
HC16_TSTD,	// Test D for Zero or Minus
HC16_TSTE,	// Test E for Zero or Minus
HC16_TSTW,	// Test for Zero or Minus Word
HC16_TSX,	// Transfer SP to X 
HC16_TSY,	// Transfer SP to Y 
HC16_TSZ,	// Transfer SP to Z 
HC16_TXKB,	// Transfer XK to B 
HC16_TXS,	// Transfer X to SP 
HC16_TXY,	// Transfer X to Y 
HC16_TXZ,	// Transfer X to Z 
HC16_TYKB,	// Transfer YK to B 
HC16_TYS,	// Transfer Y to SP 
HC16_TYX,	// Transfer Y to X 
HC16_TYZ,	// Transfer Y to Z 
HC16_TZKB,	// Transfer ZK to B 
HC16_TZS,	// Transfer Z to SP 
HC16_TZX,	// Transfer Z to X 
HC16_TZY,	// Transfer Z to Y
HC16_WAI,	// Wait for Interrupt 
HC16_XGAB,	// Exchange A with B 
HC16_XGDE,	// Exchange D with E 
HC16_XGDX,	// Exchange D with IX 
HC16_XGDY,	// Exchange D with IY 
HC16_XGDZ,	// Exchange D with IZ 
HC16_XGEX,	// Exchange E with IX 
HC16_XGEY,	// Exchange E with IY 
HC16_XGEZ,	// Exchange E with IZ 
HC16_last,
};
#endif
